#!/bin/sh

#
# Script for changing user license and 
# updating user license information for AVG 7.1 for Linux GUI
# Copyright (c) 2004, GRISOFT, Inc., www.grisoft.com
#

#
# Not for interactive use.
#
# usage: ch_license USER_NAME COMPANY [LICENSE_NUMBER] 
# 
# exit codes:
#     0: success
#    64: command line usage error
#    77: permission denied
#   252: CH_LIC_CMD exited with non-zero status when
#        trying to change license
#   253: config file writing error
#   254: config file writing error after changing license
#
# stderr output: exit status of CH_LIC_PRG
#  

usage() {
    echo "Usage: $0 USER_NAME COMPANY [LICENSE_NUMBER]"
    echo ""
    echo "Not for interactive use."
    echo "Change license and license information."    
}

#
# check privileges
#
if [ $(id -u) -ne 0 ]
then
    exit 77
fi

FILE=/opt/grisoft/avggui/config/userinfo.cfg
CH_LIC_CMD="/usr/bin/avgscan -register"

#
# Check if we can write to config 
# before changing the license number
#

if [ ! -w $FILE ]
    then
    if ! echo " " > $FILE
	then 	
	exit 253
    fi
fi
rm $FILE

#
# Parse arguments
#

if [ $# -lt 2 ]
    then
    exit 64
else
    USER_NAME=$1
    COMPANY=$2
fi

LIC_NUM="a"
if [ -n "$3" ]
    then
    LIC_NUM="$3"
fi

#
# Get license number or change license number.
#

OLDLANG=$LANG
export LANG=C

SOUT=$($CH_LIC_CMD $LIC_NUM 2>/dev/null)
STATUS=$?
export LANG=$OLDLANG

echo "SUBEXIT: $STATUS" >&2

if [ "$STATUS" -ne 0 ]
    then
    LIC_NUM=$(echo "$SOUT" | grep "license" | cut -f2 -d:)
fi

#
# Save license information
#
echo "[license_user]" > $FILE
echo "username = $USER_NAME" >> $FILE
echo "company = $COMPANY" >> $FILE
echo "license_number = $LIC_NUM" >> $FILE

if [ $? -ne 0 ]
    then
    if [[ "$STATUS" -ne 0 && -n $3 ]]
	then
	exit 253
    else
	exit 254
    fi
fi

if [[ "$STATUS" -ne 0 && -n $3 ]]
    then
    exit 252
else
    exit 0
fi
