"""
Module with main window for setting AVG GUI properties.

Copyright (C) GRISOFT, s.r.o.
"""

try:
	from sys import exit
	import pygtk
	pygtk.require('2.0')
	import gtk
	import gtk.glade

	import ConfigParser
	import os
	import sre
	import sys
	import time

	import fsh
	import config
	import cronedit
	import logger
	import messagebox
	import testres
	import PAMwrap
	import misc_widgets
	import sysinfo

except ImportError, detail:
	print detail
	exit(1)

################################################################
# CAvgGuiMainSetProperties
class CAvgGuiMainSetProperties:
	"""
	AvgMainSetProperties is class that display main window for properties setting.
	"""
	
	def __init__(self):
		self.__wTree  = gtk.glade.XML("../glade/avggui.glade")
		self.__conf   = config.CConfigStorage
		self.__cron   = cronedit.CCrontabEdit()
		self.__logger = logger.CLogger()
		self.__avg_info = sysinfo.CAvgInfo()
		
		self.__finish_cb = None
		
		# menu
		tree_store = gtk.TreeStore(gtk.gdk.Pixbuf, str)
		tree_view  = self.__wTree.get_widget("tSetProperty_menu")

		tree_view.set_model(tree_store)

		menu_column = gtk.TreeViewColumn('')

		tree_view.append_column(menu_column)

		icon_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()

		menu_column.pack_start(icon_renderer, False)
		menu_column.pack_start(text_renderer, True)

		menu_column.add_attribute(icon_renderer, 'pixbuf', 0)
		menu_column.add_attribute(text_renderer, 'text', 1)

		pic = gtk.gdk.pixbuf_new_from_file("pixmaps/avgmanagerico.png")
		parent = tree_store.append(None, [pic, "Tests"])

		pic = gtk.gdk.pixbuf_new_from_file("pixmaps/avgschedulerico.png")
		parent = tree_store.append(None, [pic, "Scheduler"])
		tree_store.append( parent, [pic, "Test schedule"])
		tree_store.append( parent, [pic, "Update schedule"])

		pic = gtk.gdk.pixbuf_new_from_file("pixmaps/avgresultsico.png")
		tree_store.append(None, [pic, "Test results"])

		pic = gtk.gdk.pixbuf_new_from_file("pixmaps/avgupdateico.png")
		tree_store.append(None, [pic, "Update"])

		pic = gtk.gdk.pixbuf_new_from_file("pixmaps/avglicense.png")
		tree_store.append(None, [pic, "License"])

		# sub frames
		sub_frame_name_list = ["vTestSet", "vTestScheduleSet", "vUpdateSchedule", "vTestResult", "vUpdSet", "vLicenseSet"]
		sub_frame_widget_list = []
		for i in sub_frame_name_list:
			w = self.__wTree.get_widget(i)
			
			sub_frame_widget_list.append(w)

		tree_view.connect("cursor-changed", self.__selected, sub_frame_widget_list)

		# reparent widget tree
		target = self.__wTree.get_widget("wSetProperties_targedBox")
		for w in sub_frame_widget_list:
			self.__reparentTree(target, w)

		self.__selected(tree_view, sub_frame_widget_list)
		
		# connect callbacks
		ok_button     = self.__wTree.get_widget("bSetProperty_ok")
		cancel_button = self.__wTree.get_widget("bSetProperty_cancel")
		apply_button  = self.__wTree.get_widget("bSetProperty_apply")
		
		main_window = self.__wTree.get_widget("wMainSetProperties")
		main_window.set_default(ok_button)
		
		#main_window.connect("delete_event", main_window.hide_on_delete)
		cancel_button.connect("clicked", self.__hide_all, main_window)
		apply_button.connect("clicked", self.__apply_button_cb)
		ok_button.connect("clicked", self.__ok_button_cb)
		
		test_res_sel_button = self.__wTree.get_widget("bTestRes_path_sel")
		test_res_sel_button.connect("clicked", self.__test_res_sel_button_cb)
		
		upd_browse_button = self.__wTree.get_widget("bUpdate_sel_path")
		upd_browse_button.connect("clicked", self.__upd_browse_button_cb)

		upd_sel_button = self.__wTree.get_widget("bUpdate_sel_files")
		upd_sel_button.connect("clicked", self.__upd_sel_button_cb)
		
		widget = self.__wTree.get_widget("rUpd_priority")
		widget.connect("toggled", self.__radio_changed_cb, 2)
		
		widget = self.__wTree.get_widget("rUpd_rcmd")
		widget.connect("toggled", self.__radio_changed_cb, 3)

		widget = self.__wTree.get_widget("rUpd_opt")
		widget.connect("toggled", self.__radio_changed_cb, 4)

		# create fsh widgets
		home_dir = os.getenv("HOME")
		self.__fsh_default = fsh.CView([home_dir], False, False)
		self.__fsh_schedul = fsh.CView([home_dir], False, False)

		scroll_window = self.__wTree.get_widget("swTest_default")
		scroll_window.add(self.__fsh_default)
		scroll_window = self.__wTree.get_widget("swTestSchedule")
		scroll_window.add(self.__fsh_schedul)

		self.__fsh_default.show()
		self.__fsh_schedul.show()

		# select days of month widget for update
		self.__dom_upd = misc_widgets.CDOMView([])
		scroll_window = self.__wTree.get_widget("swUpdSched_dom")
		scroll_window.add(self.__dom_upd)
		self.__dom_upd.show()

		# select days of week widget for update
		self.__dow_upd = misc_widgets.CDOWSelect([])
		vbox = self.__wTree.get_widget("vboxUpdSched_dow")
		vbox.pack_start(self.__dow_upd, expand = False, fill = False)
		self.__dow_upd.show_all()

		# select days of month widget for test
		self.__dom_test = misc_widgets.CDOMView([])
		scroll_window = self.__wTree.get_widget("swTestSched_dom")
		scroll_window.add(self.__dom_test)
		self.__dom_test.show()

		# select days of week widget for test
		self.__dow_test = misc_widgets.CDOWSelect([])
		vbox = self.__wTree.get_widget("vboxTestSched_dow")
		vbox.pack_start(self.__dow_test, expand = False, fill = False)
		self.__dow_test.show_all()

		self.__check_buttons = [
			('chTest_heur', 'heur', self.__conf.test_config),
			('chTest_smart', 'smart', self.__conf.test_config),
			('chTest_arc', 'arc', self.__conf.test_config),
			('chTest_rep_mac', 'macrow', self.__conf.test_config),
			('chTest_rep_pwdw', 'pwdw', self.__conf.test_config),
			('chTest_rep_ok', 'repok', self.__conf.test_config),
			('chTest_rep_ignlock', 'ignlocked', self.__conf.test_config),
			('chTest_use_default', '_use_def', self.__conf.test_config),

			('chUpd_no_diff', 'no-diff', self.__conf.update_config),
			('chUpd_no_backup', 'no-backup', self.__conf.update_config),
			('chUpd_no_log', 'no-log', self.__conf.update_config),
			('chUpd_complete', 'complete', self.__conf.update_config),

			('chUpdSched_run', 'run', self.__conf.update_sched_config),
			('chTestSched_run', 'run', self.__conf.test_sched_config),
			('chUpdSched_reboot', 'at_reboot', self.__conf.update_sched_config),
			('chTestSched_reboot', 'at_reboot', self.__conf.test_sched_config)]
			
		self.__text_entries = [
			('eLic_name', 'user_name', self.__conf.license_config),
			('eLic_company', 'company', self.__conf.license_config),
			('eLic_new_num', 'lic_num', self.__conf.license_config),
			('eTestRes_path', 'res_path', self.__conf.test_res_config),
			('eUpdate_sel_path', 'upd_path', self.__conf.update_config)]
		
		self.__spin_buttons = [
			('spTestRes_num', 'num_saved', self.__conf.test_res_config),
			('spTestRes_age', 'del_older', self.__conf.test_res_config),
			('spUpdSched_delay', 'max_delay', self.__conf.update_sched_config)]
			
		self.__radio_buttons = ["rUpd_priority", "rUpd_rcmd", "rUpd_opt"]

		self.__update_radio_val_buttons = {
			'o': 'rbUpdate_src_o',
			'f': 'rbUpdate_src_f',
			'p': 'rbUpdate_src_p',
			}
		
		for (v, w) in self.__update_radio_val_buttons.items():
			self.__wTree.get_widget(w).connect("toggled", self.__upd_radio_changed_cb, v)

		self.__upd_sched_daily_rb = ["rbUpdSched_seldays", "rbUpdSched_daily"]
		self.__test_sched_daily_rb = ["rbTestSched_seldays", "rbTestSched_daily"]		       
		

		self.__load_config()

	def __upd_sel_fill_textview(self, filelist):
		if filelist != []:
			w = self.__wTree.get_widget("txvUpdate_files")
			w.get_buffer().set_text('\n'.join(filelist))
			# adjustment has effect only after the contents of widget is rendered
			while gtk.events_pending():
				gtk.main_iteration()
			scroll_win = self.__wTree.get_widget("swUpdate_files")
			adjustment = scroll_win.get_hadjustment()
			adjustment.set_value(adjustment.upper-adjustment.page_size)

	def __upd_sel_button_cb(self, widget, data=None):
		self.__upd_sel_files_dlg = fsh.CSelPathDialog("Select Paths", self.__wTree.get_widget("wMainSetProperties"), "Select source files of offline update:")
		files = self.__upd_sel_files_dlg.get_paths(self.__conf.update_config.get_value("upd_files"), files_only=True)
		self.__upd_sel_fill_textview(files)

	def __test_res_sel_button_cb(self, widget, data=None):
		self.__test_res_sel_path_dlg = fsh.CSelOnePathDialog("Select Path", self.__wTree.get_widget("wMainSetProperties"), "Choose destination of test results:")
		old_path = self.__conf.test_res_config.get_value("res_path")
		if old_path == "":
			old_path = os.getenv("HOME")
		path = self.__test_res_sel_path_dlg.get_path(old_path)
		if path != '':
			try:
				w = self.__wTree.get_widget("eTestRes_path")
				w.set_text(path)
			except AttributeError, which:
				self.__logger.err("AttributeError: %s" % which)
				pass
		else:
			return	

	def __upd_browse_button_cb(self, widget, data=None):
		self.__upd_sel_path_dlg = fsh.CSelOnePathDialog("Select Path", self.__wTree.get_widget("wMainSetProperties"), "Select source folder of offline update:")
		path = self.__upd_sel_path_dlg.get_path(self.__conf.update_config.get_value("upd_path"))
		if path != '':
			try:
				w = self.__wTree.get_widget("eUpdate_sel_path")
				w.set_text(path)
			except AttributeError, which:
				self.__logger.err("AttributeError: %s" % which)
				pass
		else:
			return	
		
	def __confirm_deleting_results(self, parent_window, test_res_path, test_res_num=0, test_res_age=0):
		
		since_sec = time.time() - (test_res_age * 24 * 60 * 60)	
		
		try:
			res_list = testres.CTestResultsInDirList(test_res_path)
			res_list.update()
		except IOError, what:
			self.__logger.err("Path %s with stored results is not accessible.\n" % what)
			return False
	
			
		n_all_res = len(res_list.get_results_list())
		n_filtered_res = len(res_list.get_results_list(int(test_res_num), since_sec))
		n_to_delete = n_all_res - n_filtered_res
		
		if n_to_delete > 0:
			message_window = messagebox.CAvgMessageOkCancelDialog(aParent = parent_window,
				                    	   aType           = gtk.MESSAGE_WARNING,
										   aFlags          = gtk.DIALOG_MODAL,
			                        	   aMessage_format = "%d results of %d  will be deleted. Continue?" % (n_to_delete, n_all_res),
										   aLogMessage     = True)

			response = message_window.run()			
			
			if response == gtk.RESPONSE_OK:
				return True
			else:
				return False
		else:
			return True

	def __apply_button_cb(self, widget, data = None):
		main_window          = self.__wTree.get_widget("wMainSetProperties")
		test_res_path_widget = self.__wTree.get_widget('eTestRes_path')
		test_res_path        = test_res_path_widget.get_text()
		self.__finish_cb     = self.__apply_button_cb
		
		if not self.__confirm_path_writable(main_window, test_res_path):
			return

		if not self.__update_license():
			return

		test_res_num		= self.__wTree.get_widget('spTestRes_num').get_value()
		test_res_age		= self.__wTree.get_widget('spTestRes_age').get_value()	
		if not self.__confirm_deleting_results(main_window, test_res_path, test_res_num, test_res_age):
			return		

		self.__save_config()
		
	def __ok_button_cb(self, widget, data = None):
		main_window          = self.__wTree.get_widget("wMainSetProperties")
		test_res_path_widget = self.__wTree.get_widget('eTestRes_path')
		test_res_path        = test_res_path_widget.get_text()
		self.__finish_cb     = self.__ok_button_cb
		
		if not self.__confirm_path_writable(main_window, test_res_path):
			return
			
		if not self.__update_license():
			return

		test_res_num		= self.__wTree.get_widget('spTestRes_num').get_value()
		test_res_age		= self.__wTree.get_widget('spTestRes_age').get_value()	
		if not self.__confirm_deleting_results(main_window, test_res_path, test_res_num, test_res_age):
			return		
			
		self.__save_config()
		
		self.__hide_all(widget = main_window, data = main_window)
		
	def __radio_changed_cb(self, widget, data):
		self.__radio_value = data

	def __upd_radio_changed_cb(self, widget, data):
		self.__upd_src_val = data		
		
	def __load_config(self):
		try:
			# CHECK-BUTTONS
			for (w, v, c) in self.__check_buttons:
				widget = self.__wTree.get_widget(w)
				widget.set_active(c.get_value(v))

			# TEST TEXT-ENTRIES
			w = self.__wTree.get_widget("eTest_ext")
			w.set_text(";".join(self.__conf.test_config.get_value("ext")))
			w = self.__wTree.get_widget("eTest_noext")
			w.set_text(";".join(self.__conf.test_config.get_value("noext")))

			# FSHs
			p = self.__conf.test_config.get_value("def_files")
			if p != []:
				self.__fsh_default.init(p, True, False)
			
			p = self.__conf.test_sched_config.get_value("def_files")
			if p != []:
				self.__fsh_schedul.init(p, True, False)
			
			# TEXT ENTRIES
			for (w, v, c) in self.__text_entries:
				widget = self.__wTree.get_widget(w)
				widget.set_text(c.get_value(v))

			# UPDATE TEXT WIDGET
			upd_files = self.__conf.update_config.get_value("upd_files")
			self.__upd_sel_fill_textview(upd_files)

			# UPDATE SCHEDULING (day of week, day of month)
			
			# we accept only exactly one time of the day (we can except it)
			hour_list = self.__conf.update_sched_config.get_value('hours')
			if hour_list == []:
				self.__wTree.get_widget('chUpdSched_run').set_active(False)
				self.__wTree.get_widget('spUpdSched_hour').set_value(0)
			else: 
				self.__wTree.get_widget('spUpdSched_hour').set_value(hour_list[0])
			min_list = self.__conf.update_sched_config.get_value('mins')
			if min_list == []:
				self.__wTree.get_widget('chUpdSched_run').set_active(False)
				self.__wTree.get_widget('spUpdSched_min').set_value(0)	
			else: 
				self.__wTree.get_widget('spUpdSched_min').set_value(min_list[0])	
			
			dow = self.__conf.update_sched_config.get_value('days')
			dom = self.__conf.update_sched_config.get_value('d_of_mon')
			daily = 1
			if dom != []:  # empty list stands for all values
				self.__dom_upd.m_select(dom)
				daily = 0
			if dow != []:  # empty list stands for all values
				self.__dow_upd.m_select(dow)
				daily = 0

			w = self.__wTree.get_widget(self.__upd_sched_daily_rb[daily])
			w.set_active(True)

			# TEST SCHEDULING (day of week, day of month)
			
			# we accept only exactly one time of the day (we can except it)
			hour_list = self.__conf.test_sched_config.get_value('hours')
			if hour_list == []:
				self.__wTree.get_widget('chTestSched_run').set_active(False)
				self.__wTree.get_widget('spTestSched_hour').set_value(0)
			else: 
				self.__wTree.get_widget('spTestSched_hour').set_value(hour_list[0])
			min_list = self.__conf.test_sched_config.get_value('mins')
			if min_list == []:
				self.__wTree.get_widget('chTestSched_run').set_active(False)
				self.__wTree.get_widget('spTestSched_min').set_value(0)	
			else: 
				self.__wTree.get_widget('spTestSched_min').set_value(min_list[0])	
			
			dow = self.__conf.test_sched_config.get_value('days')
			dom = self.__conf.test_sched_config.get_value('d_of_mon')
			daily = 1
			if dom != []:  # empty list stands for all values
				self.__dom_test.m_select(dom)
				daily = 0
			if dow != []:  # empty list stands for all values
				self.__dow_test.m_select(dow)
				daily = 0

			w = self.__wTree.get_widget(self.__test_sched_daily_rb[daily])
			w.set_active(True)


			# SPIN BUTTONS
			for (w, v, c) in self.__spin_buttons:
				widget = self.__wTree.get_widget(w)
				widget.set_value(c.get_value(v))

			# RADIO BUTTON PRIORITY
			n = self.__conf.update_config.get_value("priority")
			self.__radio_value = n
			n = n - 2
			widget = self.__wTree.get_widget(self.__radio_buttons[n])
			widget.set_active(True)

			# RADIO BUTTON UPDATE SOURCE
			src = self.__conf.update_config.get_value("upd_src")
			self.__upd_src_val = src
			widget = self.__wTree.get_widget(self.__update_radio_val_buttons[src])
			widget.set_active(True)
			
			# LABELS
			w = self.__wTree.get_widget("lLic_num")
			w.set_text(self.__conf.license_config.get_value("lic_num"))
			
		except AttributeError, which:
			self.__logger.err("AttributeError: %s" % which)
			pass
					
	def __save_config(self):
		try:
			# CHECK-BUTTONS
			for (w, v, c) in self.__check_buttons:
				widget = self.__wTree.get_widget(w)
				c.set_value(v, widget.get_active())

			# TEST TEXT-ENTRIES
			w = self.__wTree.get_widget("eTest_ext")
			t = w.get_text()
			if t != "":
				t = sre.split(";", t)
			self.__conf.test_config.set_value("ext", t)

			w = self.__wTree.get_widget("eTest_noext")
			t = w.get_text()
			if t != "":
				t = sre.split(";", t)
			self.__conf.test_config.set_value("noext", t)

			# FSHs
			self.__conf.test_config.set_value("def_files", self.__fsh_default.get_selected())
			self.__conf.test_sched_config.set_value("def_files", self.__fsh_schedul.get_selected())
			
			# TEXT ENTRIES
			for (w, v, c) in self.__text_entries:
				widget = self.__wTree.get_widget(w)
				c.set_value(v, widget.get_text())

			# UPDATE TEXT WIDGET
			w = self.__wTree.get_widget("txvUpdate_files")
			buffer = w.get_buffer()
			files_str = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())			
			if files_str == '':
				self.__conf.update_config.set_value("upd_files", [])
			else:	
				self.__conf.update_config.set_value("upd_files", files_str.split('\n'))
			
			# UPDATE SCHEDULING

			# time (hours, mins)
			hour = self.__wTree.get_widget('spUpdSched_hour').get_value()
			min = self.__wTree.get_widget('spUpdSched_min').get_value()
			self.__conf.update_sched_config.set_value('hours', [int(hour)])
			self.__conf.update_sched_config.set_value('mins', [int(min)])

			# date (day of week, day of month)			
			w = self.__wTree.get_widget(self.__upd_sched_daily_rb[1])
			if w.get_active():
				self.__conf.update_sched_config.set_value('days', [])
				self.__conf.update_sched_config.set_value('d_of_mon', [])
			else:
				self.__conf.update_sched_config.set_value('d_of_mon', self.__dom_upd.get_list())
				self.__conf.update_sched_config.set_value('days', self.__dow_upd.get_list())

				if self.__dom_upd.get_list() + self.__dow_upd.get_list() == []:
					w = self.__wTree.get_widget("chUpdSched_run")
					w.set_active(False)
					self.__conf.update_sched_config.set_value('run', False)

			# TEST SCHEDULING

			# time (hours, mins)
			hour = self.__wTree.get_widget('spTestSched_hour').get_value()
			min = self.__wTree.get_widget('spTestSched_min').get_value()
			self.__conf.test_sched_config.set_value('hours', [int(hour)])
			self.__conf.test_sched_config.set_value('mins', [int(min)])

			# date (day of week, day of month)			
			w = self.__wTree.get_widget(self.__test_sched_daily_rb[1])
			if w.get_active():
				self.__conf.test_sched_config.set_value('days', [])
				self.__conf.test_sched_config.set_value('d_of_mon', [])
			else:
				self.__conf.test_sched_config.set_value('d_of_mon', self.__dom_test.get_list())
				self.__conf.test_sched_config.set_value('days', self.__dow_test.get_list())

				if self.__dom_test.get_list() + self.__dow_test.get_list() == []:
					w = self.__wTree.get_widget("chTestSched_run")
					w.set_active(False)
					self.__conf.test_sched_config.set_value('run', False)


			# SPIN BUTTONS
			for (w, v, c) in self.__spin_buttons:
				widget = self.__wTree.get_widget(w)
				c.set_value(v, int(widget.get_value()))

			# RADIO BUTTON
			self.__conf.update_config.set_value("priority", self.__radio_value)

			# RADIO BUTTON UPDATE SOURCE
			self.__conf.update_config.set_value("upd_src", self.__upd_src_val)
			
			self.__cron.update()
			
		except AttributeError, which:
			self.__logger.err("AttributeError: %s" % which)
			pass
			
	def __update_license(self):
		
		parent_window = self.__wTree.get_widget("wMainSetProperties")
		
		try:
			w_license   = self.__wTree.get_widget("eLic_new_num")
			w_company   = self.__wTree.get_widget("eLic_company")
			w_user_name = self.__wTree.get_widget("eLic_name")
			
			old_lic_num = self.__conf.license_config.get_value("lic_num")
			old_user_name = self.__conf.license_config.get_value("user_name")
			old_company = self.__conf.license_config.get_value("company")
			new_lic_num = w_license.get_text()
			new_user_name = w_user_name.get_text()
			new_company = w_company.get_text()

			entry_changed = False
			for (w, v, c) in self.__text_entries[:3]:
				widget = self.__wTree.get_widget(w)
				if (widget.get_text()) != (c.get_value(v)):
					entry_changed = True
					break

			if not entry_changed:
				return True

			if os.getuid() == 0:

				if old_lic_num != new_lic_num and new_lic_num != "":
					if os.system("avgscan -register %s >/dev/null 2>/dev/null" % new_lic_num):
						w_license.set_text(old_lic_num)
						
						message_window = messagebox.CAvgMessageOkDialog(aParent = parent_window,
												aType           = gtk.MESSAGE_ERROR,
												aFlags          = gtk.DIALOG_MODAL,
												aMessage_format = "License number %s is not correct." % new_lic_num,
												aLogMessage     = True)

						message_window.start()
						new_lic_num = old_lic_num
					else:
						new_lic_num = new_lic_num.split("-")[:-1]
						new_lic_num = "-".join(new_lic_num)

						# We could have saved the output of avgscan right when changing the license,
						# and thus avoid avgscan call in this function,
						# but changing license isn't time critical operation and
						# so we prefer to keep things simpler.
						self.__avg_info.update()
				else:
					new_lic_num = old_lic_num
	
				w_license.set_text(new_lic_num)
				w = self.__wTree.get_widget("lLic_num")
				w.set_label(new_lic_num)

				config = ConfigParser.ConfigParser()
				config.add_section('license_user')

				try:
					lic_config_file = open('/opt/grisoft/avggui/config/userinfo.cfg', "w")
				except IOError, filename:
					self.__logger.err("opening file: %s" % filename)


				config.set('license_user', 'username', new_user_name)
				config.set('license_user', 'company', new_company)
				config.set('license_user', 'license_number', new_lic_num)

				config.write(lic_config_file)

				return True
			
			else:
				# Authenticate and run external program using module PAMwrap

				if old_lic_num != new_lic_num and new_lic_num != "":
					update_lic = True
				else:
					update_lic = False
					new_lic_num = old_lic_num


				banner = 'Only root (superuser) can change license information.'
				info_text = 'Updating license information...'
				try:
					auth_wrap = PAMwrap.CPAMwrap('/usr/sbin/pamwrap')
				except PAMwrap.CPAMwrapError, detail:
					if detail.name == 'NO_PAM':

						w_license.set_text(old_lic_num)
						w_user_name.set_text(old_user_name)
						w_company.set_text(old_company)

						message_window = messagebox.CAvgMessageOkDialog(aParent = parent_window,
												aType           = gtk.MESSAGE_ERROR,
												aFlags          = gtk.DIALOG_MODAL,
												aMessage_format = banner + "\nCan't use PAM for root authenticaton.",
												aLogMessage     = True)

						message_window.start()
						return False
			
							
					else:
						raise
		
				ch_lic_prg = '/opt/grisoft/avggui/bin/ch_license.sh'
				try:
					if update_lic:
						# authenticate as root (using PAM) and change license	
						auth_wrap.run(parent_window, ['-o', '2', '-w', ch_lic_prg, new_user_name, new_company, new_lic_num], banner, info_text, False)
					else:
						# authenticate as root (using PAM) and change other informations than license
						auth_wrap.run(parent_window, ['-o', '2', '-w', ch_lic_prg, new_user_name, new_company], banner, info_text, False)

					self.__logger.debug("Wrapped program OK.")	
				except PAMwrap.CPAMwrapError, detail:
					self.__logger.debug("%s %s" % (detail.name, detail.msg))
					
					if detail.name == 'PROGRAM_EXIT': # wrapped program exited with non-zero status
						prg_status = auth_wrap.get_exit_status()
						if prg_status == 252 : # change license error
							pamwrap_stderr = auth_wrap.get_usr_data() 
							if pamwrap_stderr != '':
								ch_lic_prg_status = sre.match("SUBEXIT:\s*(\d+)", pamwrap_stderr).group(1)
							else:
								ch_lic_prg_status = '?'
								
							self.__logger.err("PAM-wrapped program %s error: %s"% (ch_lic_prg, prg_status))
							w_license.set_text(old_lic_num)
						
							message_window = messagebox.CAvgMessageOkDialog(aParent = parent_window,
													aType           = gtk.MESSAGE_ERROR,
													aFlags          = gtk.DIALOG_MODAL,
													aMessage_format = "License number %s is not correct." % new_lic_num,
													aLogMessage     = True)

							message_window.start()
							return False
						else:
							if prg_status == 254: # config file writing error after changing license
								msg = "License changed, but can't save config file."
							elif prg_status == 253: # config file writing error 	
								msg = "Can't save config file, license wasn't changed."
							else: # other error	
								msg = "Can't change license, %s error %s" % (ch_lic_prg, prg_status)
								
							self.__logger.err("PAM-wrapped program %s error: %d"% (ch_lic_prg, prg_status))
							w_license.set_text(old_lic_num)
							w_user_name.set_text(old_user_name)
							w_company.set_text(old_company)
							message_window = messagebox.CAvgMessageOkDialog(aParent = parent_window,
													aType           = gtk.MESSAGE_ERROR,
													aFlags          = gtk.DIALOG_MODAL,
													aMessage_format = msg,
													aLogMessage     = True)
							
							message_window.start()
							return False
						
					else: # PAM-wrapper exited with non-zero status

						info_errors = [
							'PASSWD_INVALID',
							'FIELDS_INVALID',
							'NO_USER',
							'NO_RIGHTS',
							'CANCELED',
							]
						if detail.name in info_errors:
							msg_type = gtk.MESSAGE_INFO
						else:
							msg_type = gtk.MESSAGE_ERROR
							self.__logger.err("PAM-wrapper exception: %s %s" % (detail.name, detail.msg))
						
						w_license.set_text(old_lic_num)
						w_user_name.set_text(old_user_name)
						w_company.set_text(old_company)

						message_window = messagebox.CAvgMessageOkDialog(aParent = parent_window,
												aType           = msg_type,
												aFlags          = gtk.DIALOG_MODAL,
												aMessage_format = "Authentication failed: %s." % detail.msg,
												aLogMessage     = True)
						
						message_window.start()
						return False

				else: # authentication passed, license change succesfull

					if update_lic:
						new_lic_num = new_lic_num.split("-")[:-1]
						new_lic_num = "-".join(new_lic_num)

						# We could have saved the output of avgscan right when changing the license,
						# and thus avoid avgscan call in this function,
						# but changing license isn't time critical operation and
						# so we prefer to keep things simpler.
						self.__avg_info.update()

					w_license.set_text(new_lic_num)
					w = self.__wTree.get_widget("lLic_num")
					w.set_label(new_lic_num)
				
					return True	

		except OSError, detail:
			self.__logger.err(detail)
			return False
			
	# widget, ktery se meni v zavislosti na menu bude vzdy prvni dite kontejneru			
	def __reparentTree(self, target, widget):
		childs = target.get_children()
		target.remove(childs[0])

		widget.reparent(target)
		target.reorder_child(widget, 0)
		

	def __replaceDialog(self, target,  widget):
		childs = target.get_children()
		target.remove(childs[0])

		target.add(widget)
		target.reorder_child(widget, 0)

	def __selected(self, widget, data = None):
		(path, column) = widget.get_cursor()

		new_frame = self.__wTree.get_widget("vTestSet")
		if data != None:
			if path == (0,):
				new_frame = data[0]
			elif path == (1, 0):
				new_frame = data[1]
			elif path == (1, 1):
				new_frame = data[2]
			elif path == (2,):
				new_frame = data[3]
			elif path == (3,):
				new_frame = data[4]
			elif path == (4,):
				new_frame = data[5]
			elif path == (1,):
				return
			
		self.__replaceDialog(self.__wTree.get_widget("wSetProperties_targedBox"), new_frame)
		
	def __confirm_path_writable(self, parent_window, path):

			if os.access(path, os.F_OK) == 0:
				message_window = messagebox.CAvgMessageYesNoDialog(aParent = parent_window,
				                    	   aType           = gtk.MESSAGE_QUESTION,
										   aFlags          = gtk.DIALOG_MODAL,
			                        	   aMessage_format = "Destination '%s' does not exist. Create?" % path,
										   aLogMessage     = True)

				try:
					message_window.connect('clicked', callback = self.__create_path_cb, response = gtk.RESPONSE_YES, data = path)
				except ValueError, val:
					self.__logger.err(val)

				message_window.start()

				return False

			elif os.access(path, os.R_OK | os.X_OK | os.W_OK) == 0:
				message_window = messagebox.CAvgMessageOkDialog(aParent = parent_window,
				                   aType           = gtk.MESSAGE_ERROR,
								   aFlags          = gtk.DIALOG_MODAL,
			                       aMessage_format = "You do not have write access to the destination: %s" % path,
								   aLogMessage     = True)

				message_window.start()
				
				return False

			return True
			
	def __create_path_cb(self, data = None):
		try:
			if data != None:
				self.__create_path_rec(data)

			# set new path
			w = self.__wTree.get_widget("eTestRes_path")
			w.set_text(data)
			self.__conf.test_res_config.set_value("res_path", data)
			
			# call original callback
			self.__finish_cb(self)
			
		except OSError, err:
			message_window = messagebox.CAvgMessageOkDialog(aParent = self.__wTree.get_widget("wMainSetProperties"),
				                				   aType           = gtk.MESSAGE_ERROR,
												   aFlags          = gtk.DIALOG_MODAL,
			                    				   aMessage_format = "Can not create path: %s, because %s" % (data, err),
												   aLogMessage     = True)

			message_window.start()
			
	
	def __create_path_rec(self, path):
		if not os.path.exists(path):
			dirname, tail = os.path.split(path)
			self.__create_path_rec(dirname)

		if not os.path.exists(path):
			os.mkdir(path)
			self.__logger.info("Creating path: %s" % path)
		
	def __hide_all(self, widget, data):
		data.destroy()
		
	def show(self):
		main_w = self.__wTree.get_widget("wMainSetProperties")
		main_w.show_all()
	
	
		
################################################################
# testing code
if __name__ == "__main__":

	# create new instance of gui configu
	avg_conf = config.CConfigStorage
	
	# create new config file
	try:
		avg_conf.test_config.set_value("heur", False)
		set_window  = CAvgGuiMainSetProperties()
		set_window.show()
		
		#fc = gtk.FileSelection()
		#fc.hide_fileop_buttons()
		#fc.set_select_multiple(False)
		#fc.file_list.set_sensitive(False)
		#fc.show()
		
		gtk.main()
	except IOError, what:
		print "** ERR: File IOError: %s" % what
