"""
Module for displaying information about AVG, system, license and contacts.

Copyright (C) GRISOFT, s.r.o.
"""

try:
	from sys import exit
	import config
	import sysinfo
	import logger
	
	import pygtk
	pygtk.require('2.0')
	import gtk
	import gtk.glade

except ImportError, detail:
	print detail
	exit(1)

################################################################
# CAvgGuiInfoView
class CAvgGuiInfoView:
	"""
	CAvgGuiInfoView defines window with notebook.
	"""
	
	def __init__(self, aSysInfo = None, aAvgInfo = None):
		self.__wTree  = gtk.glade.XML("../glade/avggui.glade", "wInfo")
		conf          = config.CConfigStorage

		self.__logger = logger.CLogger()
		
		if aSysInfo:
			sys_info = aSysInfo
		else:
			sys_info = sysinfo.CSysInfo()
			
		if aAvgInfo:
			avg_info = aAvgInfo
		else:
			avg_info = sysinfo.CAvgInfo()

		avg_info.connect('lic_type', self.__lic_type_change_cb)
		self.__lic_type_change_cb(avg_info.get_value('lic_type'))
		
		info_pairs = [
			("lInfo_login", "user_name"),
			("lInfo_system", "sys_name"),
			("lInfo_release", "release"),
			("lInfo_hostname", "hostname"),
			("lInfo_platform", "machine"),
			
		]
		
		avg_info_pairs = [
			("lInfo_lic_num", "lic_num"),
			("lInfo_user_name", "user_name"),
			("lInfo_company", "company"),
			("lInfo_avgver", "avgver"),
			("lInfo_aviver", "aviver")
		]
		
		try:
			for (label_name, value_name) in info_pairs:
				w = self.__wTree.get_widget(label_name)
				w.set_label(sys_info.get_value(value_name))
			
			for (label_name, value_name) in avg_info_pairs:
				w = self.__wTree.get_widget(label_name)
				w.set_label(avg_info.get_value(value_name))
				
		except AttributeError, what:
			self.__logger.err("AttributeError %s" % what)			
			pass
			

		# callbacks
		main_window = self.__wTree.get_widget("wInfo")
		main_window.connect("delete_event", self.__hide_on_delete, main_window)
		
		close_button = self.__wTree.get_widget("bInfo_close")
		close_button.connect("clicked", self.__on_close_bt_clicked, main_window)
		
		conf.license_config.connect("lic_num", self.__license_changed_cb)
		conf.license_config.connect("company", self.__company_changed_cb)
		conf.license_config.connect("user_name", self.__user_name_changed_cb)

		lic_view = self.__wTree.get_widget("tvInfo_license")
		
		f = open('/opt/grisoft/avg7/doc/license_us.txt', 'r')
		f_content = f.read()

		lic_buf = gtk.TextBuffer()
		lic_buf.insert_at_cursor(f_content)
		
		lic_view.set_buffer(lic_buf)

	def __on_close_bt_clicked(self, widget, data):
		return self.__hide_on_delete(data)
				
	def __hide_on_delete(self, widget, *args):
		widget.hide_all()
		
		return True
	
	def show(self):
		main_w = self.__wTree.get_widget("wInfo")
		main_w.show_all()
	
	def __license_changed_cb(self, new_lic_num):
		w = self.__wTree.get_widget("lInfo_lic_num")
		w.set_label(new_lic_num)
		
	def __company_changed_cb(self, new_company):
		w = self.__wTree.get_widget("lInfo_company")
		w.set_label(new_company)

	def __user_name_changed_cb(self, new_user_name):
		w = self.__wTree.get_widget("lInfo_user_name")
		w.set_label(new_user_name)
		
	def __lic_type_change_cb(self, value):		
		if value == "FREE":
			pic_file = '../glade/avgfreelogo.png'
		else:
			pic_file = '../glade/avglogo.png'
		image = self.__wTree.get_widget("imgLogoInfoProgram")
		image.set_from_file(pic_file)
		image = self.__wTree.get_widget("imgLogoInfoContacts")
		image.set_from_file(pic_file)	

################################################################
# testing code
if __name__ == "__main__":

	# create new instance of gui configu
	avg_conf = config.CConfigStorage
	
	# create new config file
	try:
		avg_conf.test_res_config.set_value("res_path", "testresults")
		avg_conf.test_res_config.set_value("num_saved", 5)
		
		info_window  = CAvgGuiInfoView()
		info_window.show()
		
		gtk.main()
	except IOError, what:
		print "** ERR: File IOError: %s" % what

