r"""Easy logger for the AVG for Linux GUI

Copyright (C) GRISOFT, Inc.
"""

try:
	import sys, os.path, threading
except Exception, detail:
	print "avggui: CRITICAL: %s" % detail
	sys.exit(1)

NONE   = 70
CRIT   = 60
ERR    = 50
WARN   = 40
NOTICE = 30
INFO   = 20
DEBUG  = 10

level_names = {
	CRIT   : "CRITICAL",
	ERR    : "ERROR",
	WARN   : "WARNING",
	NOTICE : "NOTICE",
	INFO   : "INFO",
	DEBUG  : "DEBUG"
}

class __CLogger:
	"""Class for logging."""
	
	instance = None
	logfile  = None

	def __init__(self, level = NONE):
		self.__level = level
		self.__lock = threading.Lock()


		if __file__[-4:] in ['.pyc', '.pyo']:
			self.__file_name = __file__[:-4] + '.py'
		else:
			self.__file_name = __file__
		return
	
	def __getframe(self, level=0):
		try:
			1/0
		except:
			tb = sys.exc_info()[-1]
			frame = tb.tb_frame
			while level >= 0:
				frame = frame.f_back
				level = level - 1
		return frame

	def __where(self):
		frame = None
		
		if not hasattr(sys, '_getframe'):
			frame = sys.__getframe(1)
		else:
			frame = sys._getframe(1)

		done = False
		file_name = ""
		line_num = 0
		
		while not done:
			file_name = frame.f_code.co_filename
			line_num = frame.f_lineno
			
			if file_name == self.__file_name:
				frame = frame.f_back
			else:
				done = True

		return file_name, line_num
			
	def __log(self, level, format, args):
		if level >= self.__level:
			file_name, line = self.__where()

			self.__lock.acquire()
			self.logfile.write("avggui: %s:%d: %s: %s\n" % (file_name, line, level_names[level], format))
			self.__lock.release()
		return
	
	def set_level(self, level):
		"""Set logging level."""
		
		self.__lock.acquire()
		self.__level = level
		self.__lock.release()
		return

	def get_level(self):
		"""Return logging level."""
		
		self.__lock.acquire()
		level = self.__level
		self.__lock.release()
		return level

	def log(self, level, format, *args):
		"""General logging method."""
		
		self.__log(level, format, args)
		return

	def crit(self, format, *args):
		self.__log(CRIT, format, args) 
		return
		
	def err(self, format, *args):
		self.__log(ERR, format, args)
		return
		
	def warn(self, format, *args):
		self.__log(WARN, format, args)
		return
		
	def note(self, format, *args):
		self.__log(NOTICE, format, args)
		return

	def info(self, format, *args):
		self.__log(INFO, format, args)
		return
		
	def debug(self, format, *args):
		self.__log(DEBUG, format, args)
		return

def CLogger(level = NONE, aLogfile = None):
	"""Logger is a singleton. This function returns handle to the logger object."""
	
	if __CLogger.instance == None:
		__CLogger.instance = __CLogger(level)
	
	try:
		if aLogfile:
			__CLogger.logfile = open(aLogfile, "a")
		else:
			__CLogger.logfile = sys.stdout
	except IOError, (errno, strerror):
		print "avggui: ERROR(%s): Creating log file %s: %s" % (errno, aLogfile, strerror)
	
	return __CLogger.instance

