"""
AVG7 GUI main module.

Copyright (C) GRISOFT, s.r.o.
"""

try:
	import sys
	import os
	import shutil
	import getopt
	
	import pygtk
	pygtk.require("2.0")
	import gtk
	import gtk.glade
	
	import config
	import logger
	import mainview
except ImportError, detail:
	print detail
	sys.exit(1)

def usage():
	print "Usage:\n\t %s [-l <level> | --log-level=<level>]\n" % sys.argv[0]
	
def check_locales(locales):
	return (locales != None and (locales == "C" or locales[0:2] == "en"))
	
def parse_commandline(command_line = ""):
	try:
		(opts, args) = getopt.getopt(command_line[1:], 'hl:', ['log-level=', 'help'])
	except getopt.GetoptError:
		usage()
		sys.exit(1)
	
	cmd_values = {}
	for o, v in opts:
		if o == "-h" or o == "--help":
			usage()
			sys.exit(1)
		elif o == "-l" or o == "--log-level":
			cmd_values['loglevel'] = int(v)
	
	return cmd_values

################################################################
# testing code
if __name__ == "__main__":

	# check locales
	right_locales = False
	for env_var in ["LC_ALL", "LC_MESSAGE", "LANG"]:
		right_locales = check_locales(os.getenv(env_var))
		if right_locales:
			break
	
	locale_backup = os.getenv("LANG")
	if not right_locales:
		os.putenv("LANG", "C")
		

	# check paths
	home_dir = os.getenv("HOME")
	avg7_dir = home_dir + "/.avg7"
		
	if not os.path.exists(avg7_dir):
		os.makedirs(avg7_dir)
		os.mkdir(avg7_dir + "/testresults")
		os.mkdir(avg7_dir + "/config")
	else:
		if not os.path.exists(avg7_dir + "/testresults"):
			os.mkdir(avg7_dir + "/testresults")
		
		if not os.path.exists(avg7_dir + "/config"):
			os.mkdir(avg7_dir + "/config")
		
	config_file_path_dst = avg7_dir + "/config/default.cfg" 
	config_file_path_src = "/opt/grisoft/avggui/config/default.cfg"
	
	if not os.path.exists(config_file_path_dst):
		if os.path.exists(config_file_path_src):
			shutil.copyfile(config_file_path_src, config_file_path_dst)
		else:
			print "** ERR: Cannot create default configuration file for AVG GUI"
		
	try:		
		# parse command line
		cmd_values = parse_commandline(sys.argv)

		# create logger instance
		log_level = logger.WARN
		if cmd_values.has_key('loglevel') and cmd_values['loglevel']:
			log_level = cmd_values['loglevel']
		
		logout = logger.CLogger()
		logout.set_level(log_level)

		# create new instance of gui config
		avggui_conf = config.CConfigStorage

		# load configuration
		avggui_conf_file = config.CConfigFile(config_file_path_dst, avggui_conf)
		avggui_conf_file.load()
		
		# set destination for test results
		if avggui_conf.test_res_config.get_value("res_path") == "" or not os.path.exists(avggui_conf.test_res_config.get_value("res_path")):
			if not os.path.exists(avg7_dir + "/testresults"):
				os.mkdir(avg7_dir + "/testresults")

			avggui_conf.test_res_config.set_value("res_path", avg7_dir + "/testresults")

		# incitialize main window
		main_window = mainview.CAvgGuiMainView(avggui_conf_file)

	except SystemExit, status:
		print "avggui exited with code: %s" % status
		sys.exit(status)
		
	except Exception, what:
		print "avggui: main: ERROR: AVG Initialization Error: %s" % what
		os.putenv("LANG", locale_backup)
		
		sys.exit(1)		

	try:
		gtk.gdk.threads_init()
	except RuntimeError, what:
		logout.error("%s\nYou need to install pygtk with threading support." % what)
		sys.exit(1)
	
	gtk.main()
	
	if locale_backup:
		os.putenv("LANG", locale_backup)
