"""
Module for displaying information about AVG, system, license and contacts.

Copyright (C) GRISOFT, s.r.o.
"""

try:
	from sys import exit
	import pygtk
	pygtk.require("2.0")
	import gtk
	import gtk.glade

	import os
	import sre
	import time

	import infoview
	import configview
	import logger
	import messagebox
	import testresview
	import test
	import testres
	import config
	import scanner
	import select
	import sysinfo
	import update
	import updater
except ImportError, detail:
	print detail
	exit(1)

################################################################
# CAvgGuiMainView
class CAvgGuiMainView:
	"""
	CAvgGuiMainView describes main window off application.
	"""
	
	def __init__(self, aConf_file):
		self.__wTree     = gtk.glade.XML("../glade/avggui.glade", "wMain")
		self.__conf_file = aConf_file
		self.__logger    = logger.CLogger()
		
		# callbacks
		main_window = self.__wTree.get_widget("wMain")
		main_window.connect("delete_event", self.__quit_cb)
		
		# menu items
		menu_items = {
			'programitem_quit'             : self.__quit_cb,
			'testsitem_run'                : self.__test_item_run_cb,
			'testsitem_results'            : self.__test_item_results_cb,
			'serviceitem_program_settings' : self.__service_item_settigs_cb,
			'serviceitem_update'           : self.__service_item_update_cb,
			'information_about'            : self.__information_about_cb
		}
		
		for (item, cb) in menu_items.items():
			menu_item = self.__wTree.get_widget(item)
			menu_item.connect("activate", cb)
		
		# menu check items
		conf  = config.CConfigStorage
		self.__test_item_default_chval_cb(conf.test_config.get_value('_use_def'))

		menu_item = self.__wTree.get_widget('testsitem_default')
		conf  = config.CConfigStorage
		conf.test_config.connect('_use_def', self.__test_item_default_chval_cb)
		menu_item.connect("toggled", self.__test_item_default_cb)

					
		# buttons
		button = self.__wTree.get_widget("bMain_test_res")
		button.connect("clicked", self.__test_item_results_cb)
		
		button = self.__wTree.get_widget("bMain_test_run")
		button.connect("clicked", self.__test_item_run_cb)
		
		button = self.__wTree.get_widget("bMain_update")
		button.connect("clicked", self.__service_item_update_cb)

		# info windows

		# singleton
		avg_info = sysinfo.CAvgInfo()
		avg_info.connect('lic_type', self.__lic_type_change_cb)
		self.__lic_type_change_cb(avg_info.get_value('lic_type'))
			
		self.__info_window = infoview.CAvgGuiInfoView(aAvgInfo = avg_info)

      		# save information about license to config
		conf = config.CConfigStorage
		conf.license_config.set_value("user_name", avg_info.get_value("user_name"))
		conf.license_config.set_value("company", avg_info.get_value("company"))
		conf.license_config.set_value("lic_num", avg_info.get_value("lic_num"))
		
		# info texts
		conf = config.CConfigStorage
		conf.test_res_config.connect("num_saved", self.__results_label)
		
		# status bar
		avg_info.connect('avgver', self.__avgver_change_cb)
		avg_info.connect('aviver', self.__aviver_change_cb)
		avg_info.connect('copyright', self.__copyright_change_cb)
		self.__avgver_change_cb(avg_info.get_value('avgver'))
		self.__aviver_change_cb(avg_info.get_value('aviver'))
		self.__copyright_change_cb(avg_info.get_value('copyright'))

		self.__results_label()
		self.__update_label()
		self.__test_label()
		
		# scanner callbacks
		self.scanner = scanner.CScanner()
		
		self.scanner.connect('failed', self.__test_finished)
		self.scanner.connect('terminated', self.__test_finished)
		self.scanner.connect('finished', self.__test_finished)
		
		# update callbacks
		
		self.updater = updater.CUpdater()
		
		self.updater.connect('finished', self.__update_finished)
		self.updater.connect('terminated', self.__update_finished)
		self.updater.connect('failed', self.__update_finished)

		main_window.show_all()
		
		self.__manage_license(avg_info.avgscan_output_list)
		
	def __del__(self):
		self.scanner.disconnect('failed', self.__test_finished)
		self.scanner.disconnect('terminated', self.__test_finished)
		self.scanner.disconnect('finished', self.__test_finished)

		self.updater.disconnect('finished', self.__update_finished)
		self.updater.disconnect('terminated', self.__update_finished)
		self.updater.disconnect('failed', self.__update_finished)

	def __quit_cb(self, widget = None, data = None):
		message_window = messagebox.CAvgMessageYesNoDialog(aParent = self.__wTree.get_widget("wMain"),
				                           aType           = gtk.MESSAGE_QUESTION,
										   aFlags          = gtk.DIALOG_MODAL,
			                        	   aMessage_format = "Do you realy want to quit?",
										   aLogMessage     = True)
		try:
			message_window.connect('clicked', self.__quit_yes_cb)
		except ValueError, val:
			self.__logger.err(val)

		message_window.start()
		
		return True

	def __quit_yes_cb(self):
		self.__conf_file.flush()
		gtk.main_quit()
	

	def __test_item_run_cb(self, widget, data = None):
		conf = config.CConfigStorage
		
		if conf.test_config.get_value("_use_def") and conf.test_config.get_value("def_files") != []:
			t = test.CTest()
			
			t.set_pathnames(conf.test_config.get_value("def_files"))
			t.start()
		else:
			select_window = select.CSelect()
			select_window.show()
	
	def __test_item_results_cb(self, widget, data = None):
		test_res_window = testresview.CAvgGuiTestResultsView()
		test_res_window.show()

		
	def __test_item_default_chval_cb(self, value):
		# aktualizace checkbuttonu (Default scan objects) polozce menu pri zmene hodnoty v configu
		menu_item = self.__wTree.get_widget('testsitem_default')
		menu_item.set_active(value)

	def __test_item_default_cb(self, widget, data = None):
		conf  = config.CConfigStorage
		menu_item = self.__wTree.get_widget('testsitem_default')
		conf.test_config.set_value('_use_def', menu_item.get_active())

	def __service_item_settigs_cb(self, widget, data = None):
		config_window = configview.CAvgGuiMainSetProperties()
		config_window.show()

	def __service_item_update_cb(self, widget, data = None):
		upd = update.CUpdate()
		upd.start()
		
	def __information_about_cb(self, widget, data = None):
		self.__info_window.show()
	
	def __test_label(self, data = None):
		conf  = config.CConfigStorage
		label = self.__wTree.get_widget("lMain_test_info")

		path      = conf.test_res_config.get_value("res_path")
		file_list = os.listdir(path)

		if file_list != []:
			file_list.sort()

			last_file = file_list[-1]
			clean     = last_file[-1] != "."
			mtime     = os.stat(path + "/" + last_file)[9]
			
			if clean:
				label.set_label("<b> Last test was performed on</b> %s\n and all files were <b>clean.</b>" % time.strftime("%c", time.localtime(mtime)))
			else:
				label.set_label("<b> Last test was performed on</b> %s\n and <span foreground=\"red\"><b>virus was found.</b></span>" % time.strftime("%c", time.localtime(mtime)))
	
	def __results_label(self, data = None):
		conf  = config.CConfigStorage
		
		test_res_list = testres.CTestResultsList()
		test_res_list.update()
		
		files_num = len(test_res_list.get_results_list())
		
		if not data:
			max_num = conf.test_res_config.get_value("num_saved")
		else:
			max_num = data
		
		label = self.__wTree.get_widget("lMain_test_res_info")
		label.set_label("<b>%s / %s test results saved. </b>" % (files_num, max_num))
		
	def __update_finished(self, text = None, err = None):
		self.__update_label()
		
	def __update_label(self, data = None):
		try:
			mtime = os.stat('/opt/grisoft/avg7/data/upd_vers.cfg')[9]
		except OSError:
			mtime = os.stat('/opt/grisoft/avg7/data/set_vers.cfg')[9]
		
		if mtime:
			label = self.__wTree.get_widget("lMain_update_info")
			label.set_label('<b> Last update was performed on </b>%s' % (time.strftime("%c", time.localtime(mtime))))
			
	def __test_finished(self, total, infected, errors):
		self.__results_label()
		self.__test_label()
	
	def __manage_license(self, avgscan_output):
		license_type        = ""
		expiration_text     = ""
		expired_license     = False
		
		expired_pattern        = "^(License has expired|License has expired, day of expiration)"
		not_expired_pattern    = "^(Number of days to expiration|Expiration day)"
		not_registered_pattern = "^ERROR: Not registered, please do registration with avgscan -register"
		
		main_window = self.__wTree.get_widget("wMain")
		
		message_text     = ""
		message_type     = None
		message_response = None

		for line in avgscan_output:
			if sre.match("^License type", line):
				license_type = line[:-1]

				message_text = message_text + license_type

				license_type = license_type.split(" ")[3]
				
			elif sre.match(expired_pattern, line):
				expiration_text = line[:-1]
				expired_license = True
				
				message_text = message_text + "\n" + expiration_text
				
			elif sre.match(not_expired_pattern, line):
				expiration_text = line[:-1]

				message_text = message_text + "\n" + expiration_text
				
			elif sre.match(not_registered_pattern, line):
				license_type     = "ERROR"

		if license_type == "FULL":
			if expired_license:
		
				message_type = gtk.MESSAGE_INFO
		
		elif (license_type == "TRIAL" or license_type == "SALES") and expired_license:
		
			message_response = gtk.main_quit
			message_type     = gtk.MESSAGE_ERROR
		
		elif license_type == "ERROR":
			
			message_text     = "AVG for Linux was not registred, please register it with valid license number."
			message_type     = gtk.MESSAGE_ERROR
			message_response = gtk.main_quit

		else:
			message_type     = gtk.MESSAGE_WARNING
			
		if message_type != None:
			message_window = messagebox.CAvgMessageOkDialog(aParent = main_window,
				                        	   aType           = message_type,
											   aFlags          = gtk.DIALOG_MODAL,
			                        		   aMessage_format = message_text,
											   aLogMessage     = True)
			try:
				message_window.connect('clicked', gtk.main_quit, gtk.RESPONSE_DELETE_EVENT)
				
				if message_response:
					message_window.connect('clicked', message_response)
			except ValueError, val:
				self.__logger.err(val)
			
			message_window.start()

	def __lic_type_change_cb(self, value):		
		image = self.__wTree.get_widget("imgLogoMain")
		if value == "FREE":
			pic_file = '../glade/avgfreelogo.png'
		else:
			pic_file = '../glade/avglogo.png'
		image.set_from_file(pic_file)
			
	def __avgver_change_cb(self, avgver):
		bar = self.__wTree.get_widget("stMain_avgver")
		bar.push(bar.get_context_id("avgver"), " Program version " + avgver)

	def __copyright_change_cb(self, copyright):
		bar = self.__wTree.get_widget("stMain_copyright")
		bar.push(bar.get_context_id("copyright"), copyright)

	def __aviver_change_cb(self, aviver):
		bar = self.__wTree.get_widget("stMain_aviver")
		bar.push(bar.get_context_id("aviver"), " Virus database version " + aviver)

	def show(self):
		main_w = self.__wTree.get_widget("wMain")
		main_w.show_all()

################################################################
# testing code
if __name__ == "__main__":

	# create new instance of gui configu
	avg_conf = config.CConfigStorage
	
	# create new config file
	try:
		main_window  = CAvgGuiMainView()
		main_window.show()
		
		gtk.main()
	except IOError, what:
		print "** ERR: File IOError: %s" % what

