r"""Message dialogs for the AVG for Linux GUI

Copyright (C) GRISOFT, Inc.
"""

import pygtk
pygtk.require("2.0")
import gtk

import logger

################################################################
# CAvgMessageDialog
class CAvgMessageDialog(gtk.MessageDialog):
	"""Class for displaying standard GTK Message Dialogs + logging"""
	
	def __init__(self, aParent=None, aFlags=0, aType=gtk.MESSAGE_INFO, aButtons=gtk.BUTTONS_NONE, aMessage_format=None, aLogMessage = False):
		gtk.MessageDialog.__init__(self, parent=aParent, flags=aFlags, type=aType, buttons=aButtons, message_format=aMessage_format)
		
		type2log = { gtk.MESSAGE_INFO : logger.INFO, gtk.MESSAGE_WARNING : logger.WARN,	gtk.MESSAGE_QUESTION : logger.INFO, gtk.MESSAGE_ERROR : logger.ERR }
		
		logout = logger.CLogger()

		if aLogMessage:
			logout.log(type2log[aType], aMessage_format)
		
		self.__cbs = {}
		
		gtk.MessageDialog.connect(self, 'response', self.__response_cb)

		self.__running = False
		
	def __response_cb(self, dialog = None, response = None, data1 = None):
		# call registered callbacks
		if self.__cbs.has_key(response):
			for callback, arg in self.__cbs[response]:
				if arg == None:
					callback()
				else:
					callback(arg)
		
		if not self.__running:			
			self.destroy()	
		
	def start(self):
		self.show_all()
				
	def run(self):
		self.__running = True
		response = gtk.MessageDialog.run(self)
		self.destroy()
		return response		

				
	def connect(self, name, aCallback, aResponse, data = None):
		if name != "clicked":
			raise ValueError, name
		
		if not aResponse in [gtk.RESPONSE_ACCEPT, gtk.RESPONSE_APPLY,
							gtk.RESPONSE_CANCEL, gtk.RESPONSE_CLOSE,
							gtk.RESPONSE_DELETE_EVENT, gtk.RESPONSE_HELP,
							gtk.RESPONSE_NO, gtk.RESPONSE_NONE,
							gtk.RESPONSE_OK, gtk.RESPONSE_REJECT, gtk.RESPONSE_YES]:
			raise ValueError, aResponse
		
		if self.__cbs.has_key(aResponse):
			self.__cbs[aResponse].append((aCallback, data))
		else:
			self.__cbs[aResponse] = [(aCallback, data)]
		
################################################################
# CAvgMessageOkDialog
class CAvgMessageOkDialog(CAvgMessageDialog):
	"""Class for displaying standard GTK Message Dialog with OK Button + logging"""
	
	def __init__(self, aParent=None, aFlags=0, aType=gtk.MESSAGE_INFO, aMessage_format=None, aLogMessage = False):
		CAvgMessageDialog.__init__(self, aParent, aFlags, aType, gtk.BUTTONS_OK, aMessage_format, aLogMessage)
	
	def connect(self, name, callback, response = gtk.RESPONSE_OK, data = None):
		""" Only 'clicked' event could be connected """
		
		CAvgMessageDialog.connect(self, name, callback, response, data)
		
################################################################
# CAvgMessageCancelDialog
class CAvgMessageCancelDialog(CAvgMessageDialog):
	"""Class for displaying standard GTK Message Dialog with CANCEL Button + logging"""

	def __init__(self, aParent=None, aFlags=0, aType=gtk.MESSAGE_INFO, aMessage_format=None, aLogMessage = False):
		CAvgMessageDialog.__init__(self, aParent, aFlags, aType, gtk.BUTTONS_CANCEL, aMessage_format, aLogMessage)

	def connect(self, name, callback, response = gtk.RESPONSE_CANCEL, data = None):
		""" Only 'clicked' event could be connected """

		CAvgMessageDialog.connect(self, name, callback, response, data)

################################################################
# CAvgMessageYesNoDialog
class CAvgMessageYesNoDialog(CAvgMessageDialog):
	def __init__(self, aParent=None, aFlags=0, aType=gtk.MESSAGE_INFO, aMessage_format=None, aLogMessage = False):
		CAvgMessageDialog.__init__(self, aParent, aFlags, aType, gtk.BUTTONS_YES_NO, aMessage_format, aLogMessage)

	def connect(self, name, callback, response = gtk.RESPONSE_YES, data = None):
		""" Only 'clicked' event could be connected """

		CAvgMessageDialog.connect(self, name, callback, response, data)
		

################################################################
# CAvgMessageOkCancelDialog
class CAvgMessageOkCancelDialog(CAvgMessageDialog):
	"""Class for displaying standard GTK Message Dialog with OK and Cancel Buttons + logging"""
	
	def __init__(self, aParent=None, aFlags=0, aType=gtk.MESSAGE_WARNING, aMessage_format=None, aLogMessage = False):
		CAvgMessageDialog.__init__(self, aParent, aFlags, aType, gtk.BUTTONS_OK_CANCEL, aMessage_format, aLogMessage)
	
	def connect(self, name, callback, response = gtk.RESPONSE_OK, data = None):
		""" Only 'clicked' event could be connected """
		
		CAvgMessageDialog.connect(self, name, callback, response, data)
