r"""Classes for miscelaneous widgets

CDOMStore, CDOMView -- checklist of days of the month based on gtk.ListStore and gtk.TreeView.

Copyright (C) GRISOFT, Inc.
"""


try: 
	from sys import exit
	import pygtk
	pygtk.require('2.0')
	import gtk
        import pdb
	
	import logger
except Exception, detail:
	print detail
	exit(1)

class CDOMStore(gtk.ListStore):
    """
    Contains information about selection of days of a month.

    arguments:
    sel_list -- list of integers in [1, 31] (days of month to be selected)

    methods:
    m_select -- select just the days given in the list    
    select -- (un)select given day of month
    get_list -- get the list of selected days
    
    """

    def __init__(self, sel_list=[]):
        
        gtk.ListStore.__init__(self, 'gboolean', int)

        self.__logger    = logger.CLogger()

        self.column = {
            'selected'    : 0,  # 0 => not selected, 1 => selected (viewed as checkbox)
            'day'      : 1   # number of day of month
            }

        self.__days_in_month = 31

        # fill the initial values into model
        for day in range(self.__days_in_month):
            self.append((False, day+1))

        self.m_select(sel_list)     

    def m_select(self, sel_list=[]):
        """Selects just the days given in the list"""

        iter = self.get_iter_first()
        while iter != None:
            self.set_value(iter, self.column['selected'], False)
            iter = self.iter_next(iter)
        
        for day in sel_list:
            self.select(day)

    def select(self, day, selected=True):
        """Selects given day of month"""

        if not (type(day) is int):
            raise TypeError, 'number of day must be of type int'
        if (day < 1) or (day > self.__days_in_month):
            raise ValueError, 'number of day must be lie in range [1, ' + self.__days_in_month + ']'
        self.set_value(self.get_iter(day-1), self.column['selected'], selected)

    def get_list(self):
        day_list = []
        iter = self.get_iter_first()
        while iter != None:
            if self.get_value(iter, self.column['selected']):
                day_list.append(self.get_value(iter, self.column['day']))
            iter = self.iter_next(iter)
        return day_list

class CDOMView(gtk.TreeView):
    """This class is used to display CDOMStore data."""            

    def __init__(self, sel_list):
        gtk.TreeView.__init__(self)

        self.model = CDOMStore(sel_list)
        self.set_model(self.model)
        #self.__model = CDOMStore(sel_list)
        #self.set_model(self.__model)

        self.column = gtk.TreeViewColumn("day") 

        self.renderer_sel = gtk.CellRendererToggle()
        self.renderer_day = gtk.CellRendererText()

        self.toggle_handler = self.renderer_sel.connect('toggled', self.__toggled)

        self.column.pack_start(self.renderer_sel, False)
        self.column.pack_start(self.renderer_day, False)

        self.column.add_attribute(self.renderer_sel, "active", self.model.column['selected'])
        self.column.add_attribute(self.renderer_day, "text", self.model.column['day'])

        self.append_column(self.column)

        self.set_property('headers-visible', False)
        self.set_property('rules-hint', True)

    def __toggled(self, cellrenderer, path, user_data= None):
        iter = self.model.get_iter(path)
        sel = self.model.get_value(iter, self.model.column['selected'])
        if sel:
            self.model.set_value(iter, self.model.column['selected'], False)
        else:
            self.model.set_value(iter, self.model.column['selected'], True)

    def get_list(self):            
        return self.model.get_list()

    def m_select(self, sel_list=[]):
        self.model.m_select(sel_list)

class CDOWSelect(gtk.VBox):
    """
    This class is used to select days of the week.

    arguments:
    sel_list -- list of integers in [0, 7] (days of week to be selected)

    methods:
    get_list -- get the list of selected days

    """

    # TODO: i18n and first day (Sunday/Monday?)
    
    def __init__(self, sel_list=[]):
        gtk.VBox.__init__(self, homogeneous=True)
        self.__day_names = ['Sunday',
                        'Monday',
                        'Tuesday',
                        'Wednesday',
                        'Thursday',
                        'Friday',
                        'Saturday'] 
        self.__chButtons = []
        for day in range(7):
            self.__chButtons.append(gtk.CheckButton(label=self.__day_names[day]))
            self.pack_start(self.__chButtons[day], expand=False, fill=False)

        self.m_select(sel_list)

    def m_select(self, sel_list=[]):
        """Selects just the days given in the list"""    
        for day in range(7):
            self.__chButtons[day].set_active(False)

        for day in sel_list:
            self.select(day)

    def select(self, day, active=True):
        """Selects given day of week"""

        if not (type(day) is int):
            raise TypeError, 'number of day must be of type int'
        if (day < 0) or (day > 7):
            raise ValueError, 'number of day must be lie in range [0, 7]'
        self.__chButtons[day % 7].set_active(active)

    def get_list(self):
        day_list = []
        for day in range(7):
            if self.__chButtons[day].get_active():
                day_list.append(day)
        return day_list

if __name__ == "__main__":

    w = CDOMView([3,6,31])
    w2 = CDOWSelect([0, 7, 3]) 
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window2 = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.add(w)
    window2.add(w2)
    #window.set_size_request(640,480)
    window.show_all()
    window2.show_all()
    gtk.gdk.threads_init()
    print "Selected days of month: ", w.get_list()
    print "Selected days of week:  ", w2.get_list()
    gtk.main()
    
