r"""avgscan wrapper for the AVG Linux GUI

Copyright (C) GRISOFT, Inc.
"""

try:
	from sys import exit
	import signal
	import os
	import sys
	import time
	import threading
	import sre
	import pygtk
	pygtk.require('2.0')
	import gtk
	import config
	import logger

except Exception, detail:
	print "** CRIT: %s", detail
	exit(1)

class CProcessResults(threading.Thread):
	"""Class for parsing the avgscan output."""
	
	def __init__(self, pread, report_file, callback):
		threading.Thread.__init__(self)
		self.setDaemon(False)
		
		self.__logger       = logger.CLogger()
		
		self.__pread        = pread
		self.__report_file  = report_file
		self.__callback     = callback
		self.__infected     = 0
		self.__clean        = 0
		self.__errors       = 0
		self.__total        = 0
		self.__report_clean = False
		self.__report_pwd_prot = False
		self.__report_macro = False
		self.__start_time   = None
		
		self.__progress_pattern  = "^(.*)  (Virus identified|Virus found|New variant of a known virus|Could be infected|Trojan horse|Could be a Trojan horse|Joke program|Could be a joke program|Nonsense identified|Nonsense found|Suspicion: unknown virus|May be infected by unknown virus|modified|Warning: Hidden extension|Potentially harmful program|Cannot open; not checked!|is OK|Contains password-protected files|Found some macros|Contains macros).*$"
		self.__virus_pattern     = "^(.*)  (Virus identified|Virus found|New variant of a known virus|Could be infected|Trojan horse|Could be a Trojan horse|Joke program|Could be a joke program|Nonsense identified|Nonsense found|Suspicion: unknown virus|May be infected by unknown virus|modified|Warning: Hidden extension|Potentially harmful program)(.*)$"
		self.__error_pattern     = "^(.*)  (Cannot open; not checked!.*)$"
		self.__clean_patter      = "^(.*)  (is OK).*$"
		self.__pwd_prot_pattern  = "^(.*)  (Contains password-protected files).*$"
		self.__macro_pattern    = "^(.*)  (Found some macros|Contains macros)(.*)$"
		self.__interrupt_pattern = "Test was interrupted"
		self.__failed__pattern   = "Test run failed"
		self.__successfully_finished_pattern = "Tested:"
		self.__usage_pattern = "Usage: avgscan"
		self.__free_pattern = "FREE"
		
		self.__progress_regex   = sre.compile(self.__progress_pattern)
		self.__virus_regex      = sre.compile(self.__virus_pattern)
		self.__error_regex      = sre.compile(self.__error_pattern)
		self.__clean_regex      = sre.compile(self.__clean_patter)
		self.__pwd_prot_regex   = sre.compile(self.__pwd_prot_pattern)
		self.__macro_regex      = sre.compile(self.__macro_pattern)
		self.__interrupt_regex  = sre.compile(self.__interrupt_pattern)
		self.__failed__regex    = sre.compile(self.__failed__pattern)
		self.__successfully_finished_regex = sre.compile(self.__successfully_finished_pattern)
		self.__usage_regex = sre.compile(self.__usage_pattern)
		self.__free_regex = sre.compile(self.__free_pattern)

	def __print_statistic(self, text = ""):
		if self.__report_file:
			try:
				self.__report_file.write(text)
				self.__report_file.write("\n\n\n\n")
				self.__report_file.write("------------------------------------------------------------\n")
				self.__report_file.write("Test start " + time.strftime("%c", time.localtime(time.time())) + "\n\n")
				self.__report_file.write("Elapsed time " + str(int(time.time() - self.__start_time)) + "sec.\n")
				self.__report_file.write("------------------------------------------------------------\n")
				self.__report_file.write("Scanned files  : " + str(self.__total) + "\n")
				self.__report_file.write("Infected files : " + str(self.__infected) + "\n")
				self.__report_file.write("Errors         : " + str(self.__errors) + "\n")
				self.__report_file.write("Reported files : " + str(self.__reported_count) + "\n")
				self.__report_file.write("------------------------------------------------------------\n")
				self.__report_file.flush()
			except OSError, detail:
				self.__logger.err(detail)

		return

	def report_clean_objs(self, val):
		"""Allow or disable callback functions for clean files."""
		
		self.__report_clean = val
		return

	def report_pwd_prot_objs(self, val):
		"""Allow or disable callback functions for password-protected archives."""
		
		self.__report_pwd_prot = val
		return

	def report_macro_objs(self, val):
		"""Allow or disable callback functions for files containing macros."""
		
		self.__report_macro = val
		return
		
	def run(self):
		"""Run thread to parse the avgscan output."""

		self.__start_time = time.time()
		self.__reported_count = 0
		
		try:
			results = os.fdopen(self.__pread, "r", 1)
		except Exception, detail:
			self.__logger.crit("can not read avgscan results from pipe: %s" % detail)
			return
		
		done = False

		try:
			for i in range(0, 5):
				info = results.readline()
				if self.__report_file:
					self.__report_file.write(info)
					self.__report_file.flush()

			# FREE version hasn't line with expiration		
			if not self.__free_regex.search(info):
				info = results.readline()
				if self.__report_file:
					self.__report_file.write(info)
					self.__report_file.flush()

		except OSError, detail:
			self.__logger.err(detail)

		while not done:
			try:
				line = results.readline()
			except OSError, detail:
				self.__logger.err(detail)
				line = ""

			if line == "":																# avgscan unexpectedly terminated(SIGKILL, crash or not running)
				for func in self.__callback['failed']:
					self.__print_statistic("Test run failed.")

					gtk.gdk.threads_enter()
					func(self.__total, self.__infected, self.__errors)
					gtk.gdk.threads_leave()
				
				done = True
				
			else:
				if line == "\n":
					continue

				line = sre.sub("\r", "", line[:-1])
				
				if self.__interrupt_regex.search(line):																	# test interrupted by user
					self.__print_statistic("Test was interrupted by user.")
					
					try:
						for i in range(0,4):
							line = results.readline()
					except OSError, detail:
						self.__logger.err(detail)
					
					for func in self.__callback['terminated']:
						gtk.gdk.threads_enter()
						func(self.__total, self.__infected, self.__errors)
						gtk.gdk.threads_leave()

					done = True
					
				elif self.__successfully_finished_regex.search(line):													# test finished
					self.__print_statistic()

					try:
						for i in range(0,3):
							line = results.readline()

					except OSError, detail:
						self.__logger.err(detail)

					for func in self.__callback['finished']:
						gtk.gdk.threads_enter()
						func(self.__total, self.__infected, self.__errors)
						gtk.gdk.threads_leave()

					done = True
				
				elif self.__failed__regex.search(line):																	# avgscan failed
					self.__print_statistic("Test run failed.")

					for func in self.__callback['failed']:
						gtk.gdk.threads_enter()
						func(line)
						gtk.gdk.threads_leave()

					done = True

				elif self.__usage_regex.search(line):																	# should never get here 
					self.__print_statistic()																			# usage is printed only if wrong
					try:																								# avgscan option occured
						line = results.readline()
					except OSError, detail:
						self.__logger.err(detail)

					for func in self.__callback['finished']:
						gtk.gdk.threads_enter()
						func(self.__total, self.__infected, self.__errors)
						gtk.gdk.threads_leave()

					done = True
				
				else:			
					self.__total = self.__total + 1
					
					for func in self.__callback['obj-total']:
						gtk.gdk.threads_enter()
						func(self.__total)
						gtk.gdk.threads_leave()
						
					for func in self.__callback['progress']:
						res = self.__progress_regex.search(line)
						if res:
							gtk.gdk.threads_enter()
							func(res.group(1))
							gtk.gdk.threads_leave()

					res = self.__clean_regex.search(line)																# object is clean
					if res:
						if self.__report_clean:
							self.__clean = self.__clean + 1
							for func in self.__callback['obj-clean']:
								gtk.gdk.threads_enter()
								func(self.__clean)
								gtk.gdk.threads_leave()
								
							for func in self.__callback['clean']:
								gtk.gdk.threads_enter()
								func(res.group(1))
								gtk.gdk.threads_leave()

							if self.__report_file:
								try:
									self.__report_file.write('"' + res.group(1) + '"  ' + res.group(2) + '\n')
									self.__report_file.flush()
									self.__reported_count += 1
								except OSError, detail:
									self.__logger.err(detail)

					else:
						res = self.__virus_regex.search(line)															# object is infected
						if res:
							self.__infected = self.__infected + 1
							for func in self.__callback['obj-infected']:
								gtk.gdk.threads_enter()
								func(self.__infected)
								gtk.gdk.threads_leave()
								
							for func in self.__callback['infected']:
								gtk.gdk.threads_enter()
								func(res.group(1), res.group(2) + res.group(3))
								gtk.gdk.threads_leave()

							if self.__report_file:
								try:
									self.__report_file.write('"' + res.group(1) + '"  ' + res.group(2) + res.group(3) + '\n')
									self.__report_file.flush()
									self.__reported_count += 1
								except OSError, detail:
									self.__logger.err(detail)

						else:
							res = self.__error_regex.search(line)														# error occured
							if res:
								self.__errors = self.__errors + 1
								for func in self.__callback['obj-error']:
									gtk.gdk.threads_enter()
									func(self.__errors)
									gtk.gdk.threads_leave()
								
								for func in self.__callback['error']:
									gtk.gdk.threads_enter()
									func(res.group(1), res.group(2))
									gtk.gdk.threads_leave()

								if self.__report_file:
									try:
										self.__report_file.write('"' + res.group(1) + '"  ' + res.group(2) + '\n')		# write error message to the report file
										self.__report_file.flush()
										self.__reported_count += 1
																			
									except OSError, detail:
										self.__logger.err(detail)
							else:	
								res = self.__pwd_prot_regex.search(line)																# object is password-protected archive
								if res:
									if self.__report_pwd_prot:
										# TODO: ? vytvorit jine callbacky misto clean, ikonu (?) ?
										self.__clean = self.__clean + 1
										for func in self.__callback['obj-clean']:
											gtk.gdk.threads_enter()
											func(self.__clean)
											gtk.gdk.threads_leave()
								
										for func in self.__callback['clean']:
											gtk.gdk.threads_enter()
											func(res.group(1))
											gtk.gdk.threads_leave()

										if self.__report_file:
											try:
												self.__report_file.write('"' + res.group(1) + '"  ' + res.group(2) + '\n')
												self.__report_file.flush()
												self.__reported_count += 1
											except OSError, detail:
												self.__logger.err(detail)

												
								else:	
									res = self.__macro_regex.search(line)																# object is macro
									if res:
										if self.__report_macro:
											# TODO: ? vytvorit jine callbacky misto clean, ikonu (?) ?
											self.__clean = self.__clean + 1
											for func in self.__callback['obj-clean']:
												gtk.gdk.threads_enter()
												func(self.__clean)
												gtk.gdk.threads_leave()

											for func in self.__callback['clean']:
												gtk.gdk.threads_enter()
												func(res.group(1))
												gtk.gdk.threads_leave()

											if self.__report_file:
												try:
													if self.__report_clean:
														self.__report_file.write('"' + res.group(1) + '"  ' + res.group(2) + res.group(3) + '\n')
													else:	
														self.__report_file.write('"' + res.group(1) + '"  ' + res.group(2) + '\n')
													self.__report_file.flush()
													self.__reported_count += 1
												except OSError, detail:
													self.__logger.err(detail)


									else:																						# unexpected line
										self.__logger.err("can not parse line: %s" % line)


		return
		

class __CScanner:
	"""avgscan wrapper for the AVG Linux GUI."""
	
	instance = None

	def __init__(self, pathnames = None):
		self.__conf = config.CConfigStorage
		self.__avgscan_is_running = False
		self.__report_file = ""
		self.__report_file_path = ""
		self.__avgscan_pid = None
		self.__pathnames = []
		self.__output_file = None
		
		self.__logger = logger.CLogger()

		if pathnames != None:
			self.__pathnames = pathnames

		self.__callback = {
			'infected'     : [], # object name + infected message
			'progress'     : [], # scanned object
			'clean'        : [], # object name
			'error'        : [], # object name + error message
			'obj-total'    : [], # number of total scanned objects
			'obj-infected' : [], # number of infected objects
			'obj-clean'    : [], # number of clean objects
			'obj-error'    : [], # number of errors
			'terminated'   : [], # avgscan terminated by user
			'finished'     : [], # avgscan finished
			'failed'       : []  # avgscan failed
		}

		self.connect('finished', self.__avgscan_finished_cb)
		self.connect('failed', self.__avgscan_finished_cb)
		self.connect('terminated', self.__avgscan_finished_cb)
		
	def __avgscan_finished_cb(self, total, infected, errors):
		self.__avgscan_is_running = False 
		self.__report_file.close()

		if infected:
			try:
				os.rename(self.__report_file_path, self.__report_file_path + ".") 
			except OSError, detail:
				self.__logger.err(detail)
		return

		
	def start(self):
		"""Start scanning."""

		if self.__avgscan_is_running:
			return

		self.__avgscan_is_running = True

		try:	
			pread, pwrite = os.pipe()
		except OSError, detail:
			self.__logger.err(detail)
			return

		report_dest = self.__conf.test_res_config.get_value("res_path")
		timestamp   = time.time()
		self.__report_file = None    
		self.__report_file_path = report_dest + "/testreport." + str(int(timestamp))

		try:
			self.__report_file = file(self.__report_file_path, "w")
		except OSError, detail:
			self.__report_file = None
			self.__logger.err(detail)

		try:
			self.__avgscan_pid = os.fork()
		except OSError, detail:
			self.__logger.err(detail)
			return
		
		process_results = CProcessResults(pread, self.__report_file, self.__callback)
		avgscan_options  = sre.split(" ", str(self.__conf.test_config))

		for option in avgscan_options:
			if option == "-repok":
				process_results.report_clean_objs(True)
			if option == "-pwdw":
				process_results.report_pwd_prot_objs(True)
			if option == "-macrow":
				process_results.report_macro_objs(True)

		# On Mandrake we must provide own SIGCHLD handler
		# (otherwise, os.wait() fails with zombie left)).	
		self.__saved_signal = signal.signal(signal.SIGCHLD, self.__wait)

		if not self.__avgscan_pid:

			signal.signal(signal.SIGCHLD, self.__saved_signal)
			
			try:
				os.close(pread)
			except OSError, detail:
				self.__logger.err(detail)
				return
			
			avgscan_params  = ["/opt/grisoft/avg7/bin/avgscan"] +  ["-repok", "-silent"] + avgscan_options + self.__pathnames

			self.__logger.debug("execv:" + str(avgscan_params))
			
			try:
				os.dup2(pwrite, sys.stdout.fileno())
				os.execv("/opt/grisoft/avg7/bin/avgscan", avgscan_params)
			except OSError, detail:
				self.__logger.err(detail)
				return

		else:
			try:
				os.close(pwrite)
			except OSError, detail:
				self.__logger.err(detail)

			process_results.start()
			
		return

	def __wait(self, foo, data):
		try:
			os.wait()
		except OSError, detail:
			self.__logger.err(detail)
		signal.signal(signal.SIGCHLD, self.__saved_signal)	
			
	def stop(self):
		"""Stop avgscan."""

		if self.__avgscan_is_running:
			try:
				os.kill(self.__avgscan_pid, signal.SIGINT)
			except OSError, detail:
				self.__logger.err(detail)
			self.__avgscan_is_running = False
		else:
			self.__logger.info("avgscan is not running")
		return

	def connect(self, what, func):
		"""Add callback function."""
		
		self.__callback[what].append(func)
		return

	def disconnect(self, what, func):
		"""Remove callback function."""
		
		self.__callback[what].remove(func)
		return

	def set_pathnames(self, pathnames):
		"""Set list of pathnames to scan."""
		
		self.__pathnames = pathnames
		return

def CScanner():
	"""Scanner is a singleton. This function returns handle to the scanner object."""

	if __CScanner.instance == None:
		__CScanner.instance = __CScanner()
	
	return __CScanner.instance

