r"""
Module for displaying avgscan test results. 

Copyright (C) GRISOFT, s.r.o.
"""

try:
	from sys import exit
	import config
	import testres
	import logger
	import misc
	
	import pygtk
	pygtk.require('2.0')
	import gtk
	import gtk.glade
	import gobject

	import os
	import threading
	import sre
	import time
except ImportError, detail:
	print detail
	exit(1)

################################################################
# CAvgGuiMainSetProperties
class CAvgGuiTestResultsView:
	def __init__(self):

		self.__logger = logger.CLogger()

		self.__wTree  = gtk.glade.XML("../glade/avggui.glade", "wTestResultsView")
		self.__conf   = config.CConfigStorage

		self.__flist  = testres.CTestResultsList()

		self.__ok_pic   = gtk.gdk.pixbuf_new_from_file("pixmaps/ok_result.png")
		self.__nook_pic = gtk.gdk.pixbuf_new_from_file("pixmaps/nook_result.png")

		self.__tree_store = gtk.TreeStore(gtk.gdk.Pixbuf, int)
		cell_text  = gtk.CellRendererText()
		cell_pic   = gtk.CellRendererPixbuf()
		self.__tree_store.set_sort_column_id(1, gtk.SORT_DESCENDING)

		self.__result_files_list = self.__wTree.get_widget("tTestRes_filelist")
		self.__result_files_list.set_model(self.__tree_store)

		#  popup menu to operate on test result files
		self.__result_files_popup = gtk.Menu()
		self.__delete_item = gtk.MenuItem(label = "Delete Result File")
		self.__result_files_popup.append(self.__delete_item)
		self.__delete_item.connect('activate', self.__result_files_popup_cb, 'delete')
		self.__delete_item.show()
##		selection for operations (like Delete) on more testresults files              
##		self.__result_files_list.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		
		
		self.__file_column = gtk.TreeViewColumn('Results')
		self.__result_files_list.append_column(self.__file_column)

		self.__file_column.pack_start(cell_pic, False)
		self.__file_column.pack_start(cell_text, True)

		self.__file_column.add_attribute(cell_pic, 'pixbuf', 0)
		self.__file_column.set_cell_data_func(cell_text, self.__render_time_string)		
		
		self.__file_column.set_sort_column_id(1)
		
		self.__flist.update()
		self.__result_files = self.__flist.get_results_list()

		self.__result_files_list.connect("cursor-changed", self.__selected_cb)
		self.__result_files_list.connect("button_press_event", self.__result_files_cb)
		
		self.__result_store = gtk.TreeStore(str, str)
		
		cell_path   = gtk.CellRendererText()
		cell_result = gtk.CellRendererText()
		
		self.__result_view = self.__wTree.get_widget("tTestRes_filecontent")
		self.__result_view.set_model(self.__result_store)

		path_column   = gtk.TreeViewColumn('path')
		result_column = gtk.TreeViewColumn('result')

		path_column.pack_start(cell_path, True)
		path_column.set_resizable(True)
		
		cell_path.set_property('width', 100)
		path_column.set_property('min-width', 150)
		
		result_column.pack_start(cell_result, True)

		path_column.add_attribute(cell_path, 'text', 0)
		result_column.add_attribute(cell_result, 'markup', 1)
	
		self.__result_view.append_column(path_column)
		self.__result_view.append_column(result_column)
		
		self.__result_view.set_property('rules-hint', True)
		
		# callbacks
		main_window = self.__wTree.get_widget("wTestResultsView")
		main_window.connect("delete_event", self.__delete_win_cb)
		
		close_button = self.__wTree.get_widget("bTestRes_close")
		close_button.connect("clicked", self.__close_button_cb)

		stop_button = self.__wTree.get_widget("bTestRes_stop")
		stop_button.set_sensitive(False)
		stop_button.connect("clicked", self.__stop_reader_cb)

		self.__viewed_obj_label = self.__wTree.get_widget("lTestRes_viewed_obj")

		self.__update(self.__result_files)

		self.__res_reader = None

	def __result_files_popup_cb(self, menu, item):
		if item == 'delete':

			if self.__res_reader != None:
				self.stop_reader()

			widget = self.__result_files_list	
			(path, column) = widget.get_cursor()

			if path != None:
				if self.__file_column.get_sort_order() == gtk.SORT_DESCENDING:
					index = len(self.__result_files) - path[0] - 1
				else:
					index = path[0]

				(filename, t, i) = self.__result_files[index]
				filepath = "%s/%s" % (self.__conf.test_res_config.get_value("res_path"), filename)
				
				# delete selected file
				# we don't catch errors caused by missing file
				os.unlink(filepath)
				self.__logger.info("test results file %s deleted by user" % filepath)

				# update label in main window
				self.__conf.test_res_config.set_value('num_saved', self.__conf.test_res_config.get_value('num_saved'))
				
				self.update()


	def __result_files_cb(self, treeview, event, data=None):
		if event.button != 3:
			return False
		x = int(event.x)
		y = int(event.y)
		pthinfo = treeview.get_path_at_pos(x, y)
		if pthinfo != None:
			path, col, cellx, celly = pthinfo
			treeview.grab_focus()
			if treeview.get_cursor() != (path, col):
				treeview.set_cursor( path, col, 0)
			self.__result_files_popup.popup(None, None, None, event.button, event.time)
			return True


	def __line_read_cb(self, data=None):

		(name, result) = data

		name = misc.filename_to_utf8(name)

		self.__viewed_obj = self.__viewed_obj + 1 

		if sre.match("\s*is OK", result) != None:
			self.__result_store.append(None, [name, result])
		else:
			self.__result_store.append(None, [name, "<b>" + result + "</b>"])

		self.__viewed_obj_label.set_label(str(self.__viewed_obj))


	def __read_stopped_cb(self, data=None):

		label = self.__wTree.get_widget("lTestRes_obj_to_view")
		label.set_label(label.get_label() + " (stopped, incomplete)")
		
	def __read_finished_cb(self, data=None):

		self.__res_reader = None
		self.__wTree.get_widget("bTestRes_stop").set_sensitive(False)

		#self.__result_view.set_model(self.__reading_model)


	def __load_results(self, filename, start_time = None):

		full_path = self.__conf.test_res_config.get_value("res_path") + "/" + filename

		#self.__reading_model = self.__result_view.get_model()
		#self.__result_view.set_model(None)

		self.__result_store.clear()

		# render immediately - not with threads - causes freeze
		# while gtk.events_pending():
		#	 gtk.main_iteration()

		res_file = testres.CTestResultFile(full_path)
		res_file.fill_stats()
		res_file.close()

		self.__res_reader = CResReader(full_path)
		self.__res_reader.connect('read', self.__line_read_cb)
		self.__res_reader.connect('finished', self.__read_finished_cb)
		# self.__res_reader.connect('stopped', self.__read_stopped_cb)
		self.__wTree.get_widget("lTestRes_scanned_files").set_label(str(res_file.scanned_files))
		self.__wTree.get_widget("lTestRes_scanned_sec").set_label(str(res_file.scanned_sectors))
		self.__wTree.get_widget("lTestRes_infected_files").set_label(str(res_file.infected_files))
		self.__wTree.get_widget("lTestRes_infected_sec").set_label(str(res_file.infected_sectors))
		n_obj_to_view = res_file.reported_files
		# Due to slightly diffrent format of testreslts files from avggui and from scheduled tests
		# (in scheduled tests - "Files Reported" information isn't available
		if (n_obj_to_view == None):
			n_obj_to_view = self.__get_num_of_reported(full_path)
		self.__wTree.get_widget("lTestRes_obj_to_view").set_label('/ ' + str(n_obj_to_view))
		self.__viewed_obj = 0

		self.__viewed_obj_label.set_label(str(self.__viewed_obj))
		self.__res_reader.start()
		self.__wTree.get_widget("bTestRes_stop").set_sensitive(True)

		time_label = self.__wTree.get_widget("lTestRes_test_start")

		if time_label == None:
			return

		time_label.set_label(time.strftime("%c", time.localtime(start_time)))

	def __get_num_of_reported(self, filename):
		file = open(filename, "r")
		line = file.readline()
		count = 0
		while line:
			if line[0] == '"':
				count += 1
			line = file.readline()	
		file.close()
		return count

	def __stop_reader_cb(self, data=None):	
		self.stop_reader()

	def stop_reader(self):	
		if self.__res_reader != None:
			self.__res_reader.stop()
			# to clean the file list widget properly (thread can display one obsolete line after being stopped)
			gobject.timeout_add(100, self.__sleeping_cb)
			gtk.main()
			self.__res_reader = None
			self.__read_stopped_cb()
			self.__wTree.get_widget("bTestRes_stop").set_sensitive(False)
				
 	def __sleeping_cb(self):
 		gtk.main_quit()
 		return False

	def __selected_cb(self, widget, data = None):

		if self.__res_reader != None:
			self.stop_reader()
		
		(path, column) = widget.get_cursor()

		if path != None:
			if self.__file_column.get_sort_order() == gtk.SORT_DESCENDING:
				index = len(self.__result_files) - path[0] - 1
			else:
				index = path[0]
			
			(f, t, i) = self.__result_files[index]
			self.__load_results(f, int(t))

	def __close_win(self):
		self.stop_reader()
		main_window = self.__wTree.get_widget("wTestResultsView")
		main_window.destroy()

	def __close_button_cb(self, widget, data=None):
		self.__close_win()

	def __delete_win_cb(self, widget, data=None):
		self.__close_win()
		return True

		
	def __clear_all(self):
		self.__tree_store.clear()
		self.__result_store.clear()

		time_label = self.__wTree.get_widget("lTestRes_test_start")
		time_label.set_label("")

		self.__wTree.get_widget("lTestRes_scanned_files").set_label("")
		self.__wTree.get_widget("lTestRes_scanned_sec").set_label("")
		self.__wTree.get_widget("lTestRes_infected_files").set_label("")
		self.__wTree.get_widget("lTestRes_infected_sec").set_label("")
		self.__wTree.get_widget("lTestRes_obj_to_view").set_label("")

		self.__viewed_obj_label.set_label("")

	def __update(self, result_files):
		self.__clear_all()	

		for (f, t, i) in result_files:
			seconds     = int(t)
			
			if i:
				self.__tree_store.append(None, [self.__nook_pic, seconds])
			else:
				self.__tree_store.append(None, [self.__ok_pic, seconds])

	def update(self):
		if self.__res_reader != None:
			self.stop_reader()
		self.__flist.update()
		self.__result_files = self.__flist.get_results_list()
		self.__update(self.__result_files)
				
	def show(self):

		main_w = self.__wTree.get_widget("wTestResultsView")
		main_w.show_all()

	def view_last_result(self):	
		self.__result_files_list.set_cursor(0)
			
	def __render_time_string(self, column, cell_renderer, tree_model, iter):
		seconds = tree_model.get_value(iter, 1)
		time_tuple = time.localtime(seconds)
		cell_renderer.set_property('text', time.strftime("%F, %R", time_tuple))
		return	
	
class CReadResults(threading.Thread):
	"""Class for reading saved test results"""
	
	def __init__(self, filepath, callback):

		self.__stopevent = threading.Event()
		
		threading.Thread.__init__(self)        
		self.setDaemon(False)
		self.__callback = callback
		self.__filepath = filepath

        
	def run(self):

		res_file = testres.CTestResultFile(self.__filepath)
		
		for name_result in res_file:
			#  cleaner but slower?
			#  self.__stopevent.wait(0.05)
			if self.__stopevent.isSet():
				res_file.close()
				for func in self.__callback['stopped']:                    
					gtk.gdk.threads_enter()
					func()
					gtk.gdk.threads_leave()

				return
			else:
				for func in self.__callback['read']:
					gtk.gdk.threads_enter()
					func(name_result)
					gtk.gdk.threads_leave()

		for func in self.__callback['finished']:
			gtk.gdk.threads_enter()
			func()
			gtk.gdk.threads_leave()
                    
	def stop(self):            
		self.__stopevent.set()
		#  cleaner but slower?
		#  threading.Thread.join(self)
        
class CResReader:
	"""reader of saved test results"""

	def __init__(self, filepath = None):

		self.__filepath = filepath
		self.__callback = {
			'read' : [],   # callback for line read
			'stopped' : [],   # stopped
			'finished': []   # all lines read
			}
        
	def start(self):
		self.__read_results = CReadResults(self.__filepath, self.__callback)
		self.__read_results.start()

	def connect(self, what, func):
		"""Add callback function."""
        
		self.__callback[what].append(func)
		return

	def disconnect(self, what, func):
		"""Remove callback function."""
		
		self.__callback[what].remove(func)
		return

	def stop(self):
		"""Stop reading results"""
		self.__read_results.stop()


################################################################
# testing code
if __name__ == "__main__":

	# create new instance of gui configu
	avg_conf = config.CConfigStorage
	
	# create new config file
	try:
		avg_conf.test_res_config.set_value("res_path", "testresults")
		avg_conf.test_res_config.set_value("num_saved", 5)
		
		testres_window  = CAvgGuiTestResultsView()
		testres_window.show()
		
		gtk.main()
	except IOError, what:
		print "** ERR: File IOError: %s" % what
