r"""Update window

Copyright (C) GRISOFT, Inc.
"""

try:
	from sys import exit
	import pygtk
	pygtk.require('2.0')
	import gtk
	import gtk.glade
	import sys

	import updater 
	import logger	
	import messagebox
	import sysinfo
except Exception, detail:
	print detail
	exit(1)

class CUpdate:
	def __init__(self):

		self.__logger = logger.CLogger()
		self.__avg_info = sysinfo.CAvgInfo()

		wTree = gtk.glade.XML("../glade/avggui.glade", "wUpdate")

		self.__update_window   = wTree.get_widget("wUpdate")
		self.__file_name_label = wTree.get_widget("lUpdateFileName")
		self.__file_size_label = wTree.get_widget("lUpdateFileSize")
		self.__file_per_label  = wTree.get_widget("lUpdateFilePer")
		self.__speed_label     = wTree.get_widget("lUpdateSpeed")
		self.__progress_bar    = wTree.get_widget("pUpdateProgress")
		self.__stop_button     = wTree.get_widget("bUpdateStop")
		self.__status_bar      = wTree.get_widget("sUpdateStatus")

		self.__updater = updater.CUpdater()
		self.__status_bar_contextid = self.__status_bar.get_context_id("Update_status")
	
		self.__stop_button.connect('clicked', self.__stop_update_cb)
		self.__update_window.connect('delete_event', self.__close_window_cb)
		return
	
	def __unregister_update_cbs(self):
		self.__updater.disconnect('status', self.__status_cb)
		self.__updater.disconnect('file-name', self.__file_name_cb)
		self.__updater.disconnect('file-dlnow', self.__file_dlnow_cb)
		self.__updater.disconnect('file-per', self.__file_percents_cb)
		self.__updater.disconnect('total-per', self.__total_percents_cb)
		self.__updater.disconnect('speed', self.__speed_cb)
		self.__updater.disconnect('eta', self.__eta_cb)
		self.__updater.disconnect('terminated', self.__terminated_cb)
		self.__updater.disconnect('finished', self.__finished_cb)
		self.__updater.disconnect('failed', self.__failed__cb)
		
		return

	def __stop_update_cb(self, user_data = None):
		self.__updater.stop()
		return

	def __close_window_cb(self, widget, event, data = None):
		self.__updater.stop()
		return 

	def __status_cb(self, message):
		self.__status_bar.pop(self.__status_bar_contextid)
		self.__status_bar.push(self.__status_bar_contextid, message)
		return

	def __file_name_cb(self, name):
		self.__file_name_label.set_label(name)
		return
	
	def __file_dlnow_cb(self, size):
		self.__file_size_label.set_label(size)
		return
	
	def __file_percents_cb(self, per):
		self.__file_per_label.set_label(per)
		return

	def __total_percents_cb(self, per):
		self.__progress_bar.set_fraction(float(per)/100.0)
		return
	
	def __speed_cb(self, speed):
		self.__speed_label.set_label(speed)
		return
	
	def __eta_cb(self, eta):
		self.__progress_bar.set_text(eta)
		return

	def __terminated_cb(self):
		self.__unregister_update_cbs()

		message_window = messagebox.CAvgMessageOkDialog(aParent = self.__update_window,
				                   aType           = gtk.MESSAGE_WARNING,
								   aFlags          = gtk.DIALOG_MODAL,
			                       aMessage_format = "Update was terminated by user.",
								   aLogMessage     = True)

		try:
			message_window.connect('clicked', self.__update_window.destroy)
		except ValueError, val:
			self.__logger.err(val)

		message_window.start()

		return
		
	def __finished_cb(self, text):
		self.__unregister_update_cbs()

		message_window = messagebox.CAvgMessageOkDialog(aParent = self.__update_window,
				                   aType           = gtk.MESSAGE_INFO,
								   aFlags          = gtk.DIALOG_MODAL,
			                       aMessage_format = text,
								   aLogMessage     = True)

		try:
			message_window.connect('clicked', self.__update_window.destroy)
		except ValueError, val:
			self.__logger.err(val)

		message_window.start()

		self.__avg_info.update()

		return

	def __failed__cb(self, text, err):
		self.__unregister_update_cbs()

		message_text   = "%s.\nReason: %s" % (text, err)
		message_window = messagebox.CAvgMessageOkDialog(aParent = self.__update_window,
				                   aType           = gtk.MESSAGE_ERROR,
								   aFlags          = gtk.DIALOG_MODAL,
			                       aMessage_format = message_text,
								   aLogMessage     = True)

		try:
			message_window.connect('clicked', self.__update_window.destroy)
		except ValueError, val:
			self.__logger.err(val)

		message_window.start()

		return

	def start(self):
		self.__updater.connect('status', self.__status_cb)
		self.__updater.connect('file-name', self.__file_name_cb)
		self.__updater.connect('file-dlnow', self.__file_dlnow_cb)
		self.__updater.connect('file-per', self.__file_percents_cb)
		self.__updater.connect('total-per', self.__total_percents_cb)
		self.__updater.connect('speed', self.__speed_cb)
		self.__updater.connect('eta', self.__eta_cb)
		self.__updater.connect('terminated', self.__terminated_cb)
		self.__updater.connect('finished', self.__finished_cb)
		self.__updater.connect('failed', self.__failed__cb)

		self.__update_window.show_all()
		self.__updater.start()
	

if __name__ == "__main__":
	update = CUpdate()
	update.start()
	gtk.gdk.threads_init()
	gtk.main()
