r"""avgupdate wrapper for the AVG Linux GUI

Copyright (C) GRISOFT, Inc.
"""

try:
	from sys import exit
	import signal
	import os
	import sys
	import threading
	import sre
	import pygtk
	pygtk.require('2.0')
	import gtk
	import config
	import logger
except Exception, detail:
	print "** CRIT: %s", detail
	exit(1)

class CProcessResults(threading.Thread):
	"""Class for parsing the avgupdate output."""
	
	def __init__(self, pread, callback):
		threading.Thread.__init__(self)
		self.setDaemon(False)
		
		self.__logger        = logger.CLogger()
		
		self.__pread         = pread
		self.__callback      = callback
		self.__ctf_file_pattern = "\.ctf"

		self.__error_message = ""
		self.__progress_pattern = "^(.*)%\s\[(.*)\s(.*%)\s(.*/.*)\]\s(.*)\s(.*)$"
		self.__file_info_pattern = "^Downloading\sfile:\s(.*)\s*\[\s*(.*)\]$"
		self.__ignore_pattern = "(^\s*$|File  : .+ not modified\.)"
		self.__interrupt_pattern = "Interrupt"
		self.__error_message_pattern = "(Can not open log file|Can not find value in config file for field|Can not parse config file|Only root can do that|Process 'avgupdate' is already running|Can not download the file|Error occurred while processing the update files|Error occurred while parsing the CTF file|File .* not found or damaged.|Can not create directory|Can not copy file|Can not delete directory|Can not create file|File .* not found)"
		self.__status_pattern = "(Preparing to download files from|Updating|Starting AVG Anti-Virus Daemons|AVG Anti-Virus Daemons start failed|Stopping AVG Anti-Virus Daemons|Sending signal to AVG Anti-Virus Daemons - reload virus database|Can not send signal to AVG Anti-Virus Daemons - reload virus database|Can not stop AVG Anti-Virus Daemons)"
		self.__successfully_finished_pattern = "(Nothing new to update|Online update finished successfully|Offline update finished successfully|Update files downloaded successfully)"
		self.__failed__pattern = "(Online update failed|Offline update failed|Download failed)"

		self.__progress_regex = sre.compile(self.__progress_pattern)
		self.__file_info_regex = sre.compile(self.__file_info_pattern)
		self.__ignore_regex = sre.compile(self.__ignore_pattern)
		self.__interrupt_regex = sre.compile(self.__interrupt_pattern)
		self.__error_message_regex = sre.compile(self.__error_message_pattern)
		self.__status_regex = sre.compile(self.__status_pattern)
		self.__successfully_finished_regex = sre.compile(self.__successfully_finished_pattern)
		self.__failed__regex = sre.compile(self.__failed__pattern)
		self.__ctf_file_regex = sre.compile(self.__ctf_file_pattern)

	def run(self):
		"""Run thread to parse the avgupdate output."""
		
		try:
			results = os.fdopen(self.__pread, "r", 1)
		except Exception, detail:
			self.__logger.crit("can not read avgscan results from pipe: %s" % detail)
			return
			
		done = False
		line = ""

		while not done:
			try:
				line = results.readline()
			except OSError, detail:
				self.__logger.crit(detail)
				line = ""
			
			if line == "":															# avgupdate was unexpectedly terminated(SIGKILL, crash or not running)
				for func in self.__callback['failed']:
					gtk.gdk.threads_enter()
					if self.__error_message != "":
						func("Update process failed.", "%s" % self.__error_message)
					else:
						func("Update process failed.", "Unexpectedly terminated or not running.")
					gtk.gdk.threads_leave()
				
				done = True

			else:
				line = line[:-1]
				
				res = self.__progress_regex.search(line)
				if res:
					if not self.__ctf_file_regex.search(res.group(2).rstrip()) :
						for func in self.__callback['total-per']:
							gtk.gdk.threads_enter()
							func(res.group(1))
							gtk.gdk.threads_leave()

						for func in self.__callback['eta']:
							gtk.gdk.threads_enter()
							func(res.group(6))
							gtk.gdk.threads_leave()
						
					for func in self.__callback['file-per']:
						gtk.gdk.threads_enter()
						func(res.group(3))
						gtk.gdk.threads_leave()
						
					for func in self.__callback['file-dlnow']:
						gtk.gdk.threads_enter()
						func(res.group(4))
						gtk.gdk.threads_leave()
						
					for func in self.__callback['speed']:
						gtk.gdk.threads_enter()
						func(res.group(5))
						gtk.gdk.threads_leave()
						
				else:
					res = self.__file_info_regex.search(line)
					if res:
						for func in self.__callback['file-name']:
							gtk.gdk.threads_enter()
							func(res.group(1))
							gtk.gdk.threads_leave()
						
						for func in self.__callback['file-size']:
							gtk.gdk.threads_enter()
							func(res.group(2))
							gtk.gdk.threads_leave()

						if not self.__ctf_file_regex.search(res.group(1).rstrip()) :
							for func in self.__callback['status']:
								gtk.gdk.threads_enter()
								func("Downloading update files...")
								gtk.gdk.threads_leave()
						else:
							for func in self.__callback['status']:
								gtk.gdk.threads_enter()
								func("Downloading control file...")
								gtk.gdk.threads_leave()


					else:
						res = self.__interrupt_regex.search(line)
						if res:
							for func in self.__callback['terminated']:						# avgupdate terminated by user(SIGTERM or SIGINT)
								gtk.gdk.threads_enter()
								func()
								gtk.gdk.threads_leave()

							done = True

						else:
							res = self.__status_regex.search(line)
							if res:
								for func in self.__callback['status']:
									gtk.gdk.threads_enter()
									func(line)
									gtk.gdk.threads_leave()

							else:
								res = self.__successfully_finished_regex.search(line)		# avgupdate successfully finished
								if res:
									for func in self.__callback['finished']:
										gtk.gdk.threads_enter()
										func(line)
										gtk.gdk.threads_leave()
									
									done = True

								else:
									res = self.__failed__regex.search(line)					# avgupdate failed
									if res:
										for func in self.__callback['failed']:
											gtk.gdk.threads_enter()
											func(line, self.__error_message)
											gtk.gdk.threads_leave()

										done = True

									else:
										res = self.__ignore_regex.search(line)				# ignore empty lines
										if res:
											pass
										else:
											res = self.__error_message_regex.search(line)	# save errror message
											if res:
												self.__error_message = line

											else:											# unexpected line
												self.__logger.warn("Can not parse line: %s" % line)



		try:
			results.close()
		except OSError, detail:
			self.__logger.err(detail)

		return

class __CUpdater:
	"""avgupdate wrapper for the AVG Linux GUI."""
	
	instance = None

	def __init__(self):
		self.__conf = config.CConfigStorage
		self.__avgupdate_is_running = False
		self.__avgupdate_pid = None
		self.__avgupdate_path = "/opt/grisoft/avg7/bin/avgupdate"
		
		self.__logger = logger.CLogger()

		self.__callback = {
			'status'       : [], # text for the status bar
			'file-name'    : [], # file name
			'file-size'    : [], # file size
			'file-dlnow'   : [], # downloaded size
			'file-per'     : [], # file percents
			'total-per'    : [], # total percents
			'speed'        : [], # actual speed
			'eta'          : [], # estimated time of arrival 
			'terminated'   : [], # avgupdate terminated by user
			'finished'     : [], # avgupdate finished successfully
			'failed'       : []  # avgupdate failed
		}

		self.connect('finished', self.__avgupdate_finished_cb)
		self.connect('failed', self.__avgupdate_finished_cb)
		self.connect('terminated', self.__avgupdate_finished_cb)

		return
	
	def __avgupdate_finished_cb(self, text = None, err = None):
		self.__avgupdate_is_running = False
		return

	def start(self):
		"""Start update process."""
		
		if not os.path.exists(self.__avgupdate_path):
			for func in self.__callback['failed']:
				func("Update process failed.", "File '%s' does not exist." % self.__avgupdate_path)
			self.__logger.err("File '%s' does not exist." % self.__avgupdate_path)
			return

		if not os.access(self.__avgupdate_path, os.X_OK):
			for func in self.__callback['failed']:
				func("Update process failed.", "Sorry, you do not have permission to execute avgupdate.")
			self.__logger.err("You do not have permission to execute avgupdate.\nNote: you have to be member of group, which have permission to execute avgupdate process.")
			return
		
		if self.__avgupdate_is_running:
			self.__logger.warn("update process is already running")
			return

		self.__avgupdate_is_running = True

		try:	
			pread, pwrite = os.pipe()
		except OSError, detail:
			self.__logger.err(detail)
			return

		try:
			self.__avgupdate_pid = os.fork()
		except OSError, detail:
			self.__logger.crit(detail)
			return

		process_results = CProcessResults(pread, self.__callback)

		# On Mandrake we must provide own SIGCHLD handler
		# (otherwise, os.wait() fails with zombie left)).	
		self.__saved_signal = signal.signal(signal.SIGCHLD, self.__wait)

		if not self.__avgupdate_pid:

			signal.signal(signal.SIGCHLD, self.__saved_signal)
			
			try:
				os.close(pread)
			except OSError, detail:
				self.__logger.err(detail)
				return
			
			avgupdate_options = sre.split(" ", str(self.__conf.update_config))
			avgupdate_params = [self.__avgupdate_path] +  ["-g"] + avgupdate_options
			
			try:
				os.dup2(pwrite, sys.stdout.fileno())									# redirect stdout and stderr to the pipe
				os.dup2(pwrite, sys.stderr.fileno())
				os.execv(self.__avgupdate_path, avgupdate_params)						# start process with avgupdate
			except OSError, detail:
				self.__logger.crit(detail)
				return

		else:
			try:
				os.close(pwrite)
			except OSError, detail:
				self.__logger.err(detail)

			process_results.start()														# start thread to process output from avgupdate
			
		return

	def stop(self):
		"""Stop avgupdate."""

		if self.__avgupdate_is_running:
			try:
				os.kill(self.__avgupdate_pid, signal.SIGINT)
			except OSError, detail:
				self.__logger.crit(detail)
			self.__avgupdate_is_running = False
		else:
			self.__logger.warn("avgupdate is not running")
		return

	def connect(self, what, func):
		"""Add callback function."""
		
		self.__callback[what].append(func)
		return

	def disconnect(self, what, func):
		"""Remove callback function."""
		
		self.__callback[what].remove(func)
		return
	
	def __wait(self, foo, data):
		try:
			os.wait()
		except OSError, detail:
			self.__logger.err(detail)
		signal.signal(signal.SIGCHLD, self.__saved_signal)	
	

def CUpdater():
	"""Updater is a singleton. This function returns handle to the updater object."""
	
	if __CUpdater.instance == None:
		__CUpdater.instance = __CUpdater()
	
	return __CUpdater.instance

